@@ -1,18 +1,17 @@
-/* $Header: /fridge/cvs/xscorch/sai/saibuy.c,v 1.22 2001/08/15 04:55:57 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sai/saibuy.c,v 1.29 2004/02/26 06:34:54 justins Exp $ */
 /*
 
-   xscorch - sai.c            Copyright(c) 2001,2000 Justin David Smith
-                              Copyright(c) 2001 Jacob Luna Lundberg
+   xscorch - sai.c            Copyright(c) 2000-2003 Justin David Smith
+                              Copyright(c) 2001-2003 Jacob Luna Lundberg
    justins(at)chaos2.org      http://chaos2.org/
    jacob(at)chaos2.org        http://chaos2.org/~jacob
 
-   Main file for AI code
+   AI purchasing code
 
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,10 +24,11 @@
 
 */
 #include <saiint.h>              /* AI internal header */
+
+#include <sgame/sconfig.h>       /* Need basic configuration */
 #include <sgame/seconomy.h>      /* Need current economical state */
 #include <sgame/sinventory.h>    /* Need to get player inventory */
 #include <sgame/splayer.h>       /* Need player data */
-#include <sgame/sconfig.h>       /* Need basic configuration */
 #include <sgame/sweapon.h>       /* Yield calculations */
 #include <sgame/swindow.h>       /* Used for status messages */
 
@@ -38,15 +38,15 @@ static inline int _sc_ai_budget(const sc_config *c, const sc_player *p) {
 /* sc_ai_budget
    Calculate the amount of money the AI is permitted to spend this turn */
 
-   const sc_economy *ec;   /* Economy data */
-   double percent;         /* Percentage of rounds played */
+   const sc_economy_config *ec; /* Economy data */
+   double percent;              /* Percentage of rounds played */
 
    /* Any budget constraints? */
    if(c->aicontrol->nobudget) {
       return(p->money * 0.75);
    }
 
-   ec = c->economy;
+   ec = c->economics;
    if(ec->computersbuy) {
       /* Only buy based on current interest rate, or the default
          conservative budget, whichever is larger */
@@ -63,9 +63,9 @@ static inline int _sc_ai_budget(const sc_config *c, const sc_player *p) {
       percent += ((double)(c->curround + 1) / c->numrounds) * (1 - percent);
       if(percent > 1) percent = 1;
 
-      #if SC_AI_DEBUG_BUY
+      if(SC_AI_DEBUG_BUY) {
          printf("AI_budget:   %s, %s has budget %d\n", sc_ai_name(p->aitype), p->name, (int)(p->money * percent));
-      #endif /* Debug? */
+      }
 
       /* That ought to be sufficiently devastating. */
       return(p->money * percent);
@@ -90,19 +90,19 @@ static bool _sc_ai_buy_last_weapons(const sc_config *c, sc_player *p, int *budge
    bool result = false;
 
    for(outer = SC_AI_BUY_MAX_OF_WEAPON; outer > 0; --outer) {
-      info = sc_weapon_last(c->weapons, SC_WEAPON_LIMIT_ALL);
+      info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_REVERSE);
       for(inner = count; inner > 0; --inner) {
          if(sc_inventory_can_buy_weapon(p, info, *budget) &&
             sc_weapon_statistic(c->weapons, info, p, SC_WEAPON_STAT_YIELD) > 0) {
             /* Buy some of this weapon. */
             sc_inventory_buy_weapon(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             result = true;
          } /* Can we buy this weapon? */
-         info = sc_weapon_prev(c->weapons, info, SC_WEAPON_LIMIT_ALL);
+         info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_REVERSE);
       } /* Iteration on info lists */
    } /* Iterate multiples of weapons */
 
@@ -158,9 +158,9 @@ static bool _sc_ai_buy_weapons_from_list(const sc_config *c, sc_player *p, int *
             /* buy some of this weapon. */
             sc_inventory_buy_weapon(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             result = true;
          } /* Can we buy this weapon? */
       } /* Buying weapons */
@@ -195,11 +195,11 @@ static bool _sc_ai_buy_weapons_by_score(const sc_config *c, sc_player *p, int *b
    }
 
    /* Calculate scores, and set initial ordering */
-   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL);
+   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    for(i = 0; i < count; ++i) {
       ordering[i] = info->ident;
       scores[i] = sc_weapon_statistic(c->weapons, info, p, SC_WEAPON_STAT_YIELD);
-      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL);
+      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    }
 
    /* Bubble-sort the scores */   
@@ -240,11 +240,11 @@ static bool _sc_ai_buy_weapons_by_weighted_score(const sc_config *c, sc_player *
    }
 
    /* Calculate scores, and set initial ordering */
-   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL);
+   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    for(i = 0; i < count; ++i) {
       ordering[i] = info->ident;
       scores[i] = sc_weapon_statistic(c->weapons, info, p, SC_WEAPON_STAT_ECON_YIELD);
-      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL);
+      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    }
 
    /* Bubble-sort the scores */   
@@ -286,11 +286,11 @@ static bool _sc_ai_buy_precision_weapons(const sc_config *c, sc_player *p, int *
    }
 
    /* Calculate scores, and set initial ordering */
-   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL);
+   info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    for(i = 0; i < count; ++i) {
       ordering[i] = info->ident;
       scores[i] = sc_weapon_statistic(c->weapons, info, p, SC_WEAPON_STAT_PRECISION_YIELD);
-      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL);
+      info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
    }
 
    /* Bubble-sort the scores */   
@@ -307,6 +307,43 @@ static bool _sc_ai_buy_precision_weapons(const sc_config *c, sc_player *p, int *
 
 
 
+static bool _sc_ai_buy_shield_sappers(const sc_config *c, sc_player *p, int *budget) {
+/* sc_ai_buy_shield_sappers
+   Buy up Shield Sappers, which have no yield,
+   but AIs can use them against shielded players. */
+
+   sc_weapon_info *info;        /* Weapon information */
+   int count = sc_weapon_count(c->weapons, SC_WEAPON_LIMIT_ALL);
+   int sapbudget = *budget / 4; /* Sappers shouldn't drain the budget */
+   int inner, outer;            /* Iterators */
+   bool result = false;
+
+/* TEMP - We're not using this for now.  See the comment in the
+   shield sappers section of saiturn.c for more information...  */
+return(result);
+
+   for(outer = SC_AI_BUY_MAX_OF_WEAPON; outer > 0; --outer) {
+      info = sc_weapon_first(c->weapons, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_REVERSE);
+      for(inner = count; inner > 0; --inner) {
+         if(sc_inventory_can_buy_weapon(p, info, sapbudget) && SC_WEAPON_IS_SAPPER(info)) {
+            /* Buy some of this weapon. */
+            sc_inventory_buy_weapon(p, info);
+            *budget -= info->price;
+            if(SC_AI_DEBUG_BUY) {
+               printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
+            }
+            result = true;
+         } /* Can we buy this weapon? */
+         info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_REVERSE);
+      } /* Iteration on info lists */
+   } /* Iterate multiples of weapons */
+
+   return(result);
+
+}
+
+
+
 static bool _sc_ai_buy_auto_defense(const sc_config *c, sc_player *p, int *budget) {
 /* sc_ai_buy_auto_defense
    Buy auto-defense, only if this is a sequential game. */
@@ -318,7 +355,7 @@ static bool _sc_ai_buy_auto_defense(const sc_config *c, sc_player *p, int *budge
    if(c->options.mode == SC_CONFIG_MODE_SYNCHRONOUS) return(false);
 
    /* If this is a sequential game, make damn sure we buy autodefense */
-   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL);
+   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    for(; count > 0; --count) {
       if(sc_inventory_can_buy_accessory(p, info, *budget)) {
          /* Make sure the damn thing is auto-defence. */
@@ -326,13 +363,13 @@ static bool _sc_ai_buy_auto_defense(const sc_config *c, sc_player *p, int *budge
             /* Buy some of this autodefense */
             sc_inventory_buy_accessory(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             return(true);
          } /* Is accessory auto-defense? */
       } /* Can we buy? */
-      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
+      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    } /* Iterate through */
 
    return(false);
@@ -352,7 +389,7 @@ static bool _sc_ai_buy_contact_triggers(const sc_config *c, sc_player *p, int *b
    if(!c->weapons->tunneling) return(false);
 
    /* Buy contact triggers if we can. */
-   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL);
+   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    for(; count > 0; --count) {
       /* Make sure the damn thing is a contact trigger. */
       if(SC_ACCESSORY_IS_TRIGGER(info)) {
@@ -360,13 +397,13 @@ static bool _sc_ai_buy_contact_triggers(const sc_config *c, sc_player *p, int *b
             /* Buy some of these contact triggers */
             sc_inventory_buy_accessory(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             return(true);
          } /* Can we buy? */
       } /* Is accessory a contact trigger? */
-      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
+      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    } /* Iterate through */
 
    return(false);
@@ -385,7 +422,7 @@ static bool _sc_ai_buy_last_shields(const sc_config *c, sc_player *p, int *budge
    int count = sc_accessory_count(c->accessories, SC_ACCESSORY_LIMIT_ALL);
 
    /* Buy some shielding */
-   info = sc_accessory_last(c->accessories, SC_ACCESSORY_LIMIT_ALL);
+   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_REVERSE);
    for(; count > 0; --count) {
       if(sc_inventory_can_buy_accessory(p, info, *budget)) {
          /* Make sure the damn thing is a shield. */
@@ -393,13 +430,13 @@ static bool _sc_ai_buy_last_shields(const sc_config *c, sc_player *p, int *budge
             /* Buy some of this shield */
             sc_inventory_buy_accessory(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             return(true);  /* That's all, for now */
          } /* Is accessory a shield? */
       } /* Can we buy? */
-      info = sc_accessory_prev(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
+      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_REVERSE);
    } /* Iterate up the accessory list. */
 
    return(false);
@@ -451,9 +488,9 @@ static bool _sc_ai_buy_shields_from_list(const sc_config *c, sc_player *p, int *
             /* buy some of this shielding. */
             sc_inventory_buy_accessory(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             return(true);  /* That's all, for now */
          } /* Can we buy these shields? */
       } /* Is this accessory a shield? */
@@ -487,14 +524,17 @@ static bool _sc_ai_buy_best_shields(const sc_config *c, sc_player *p, int *budge
    }
 
    /* Calculate scores, and set initial ordering */
-   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL);
+   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    for(i = 0; i < count; ++i) {
       ordering[i] = info->ident;
       /* TEMP HACK - This should call a function in sgame/saccessory.c like the weapon version does. -JL */
+      /* TEMP NOTE ABOUT MY TEMP HACK - Actually maybe sgame/sshield.c would make more sense, heh... */
       scores[i] = info->shield;  /* = sc_accessory_yield(c->accessories, info); */
-      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
+      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    }
 
+   /* TEMP NOTE NO. 2 - Ideally we should decide what's best based on type as well...
+                        For example, a spreader probably prefers standard shielding.  ? */
    /* Bubble-sort the scores */
    _sc_ai_sorted_accessory_scores(c->accessories, ordering, scores);
 
@@ -517,7 +557,7 @@ static bool _sc_ai_buy_batteries(const sc_config *c, sc_player *p, int *budget)
    int count = sc_accessory_count(c->accessories, SC_ACCESSORY_LIMIT_ALL);
 
    /* Buy batteries if we can. */
-   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL);
+   info = sc_accessory_first(c->accessories, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    for(; count > 0; --count) {
       /* Make sure the damn thing is a battery. */
       if(SC_ACCESSORY_IS_BATTERY(info)) {
@@ -525,13 +565,13 @@ static bool _sc_ai_buy_batteries(const sc_config *c, sc_player *p, int *budget)
             /* Buy some of these here batteries */
             sc_inventory_buy_accessory(p, info);
             *budget -= info->price;
-            #if SC_AI_DEBUG_BUY
+            if(SC_AI_DEBUG_BUY) {
                printf("   AI_buy:   %s, %s bought %s.  $%d\n", sc_ai_name(p->aitype), p->name, info->name, *budget);
-            #endif /* Debug? */
+            }
             return(true);
          } /* Can we buy? */
       } /* Is accessory a battery? */
-      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
+      info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL | SC_ACCESSORY_SCAN_FORWARD);
    } /* Iterate through */
 
    return(false);
@@ -582,6 +622,7 @@ sc_ai_result sc_ai_player_buy(const sc_config *c, sc_player *p) {
          _sc_ai_buy_contact_triggers(c, p, &budget);
          _sc_ai_buy_auto_defense(c, p, &budget);
          _sc_ai_buy_best_shields(c, p, &budget);
+         _sc_ai_buy_shield_sappers(c, p, &budget);
          _sc_ai_buy_weapons_by_weighted_score(c, p, &budget);
          break;
 
@@ -591,6 +632,7 @@ sc_ai_result sc_ai_player_buy(const sc_config *c, sc_player *p) {
          _sc_ai_buy_auto_defense(c, p, &budget);
          _sc_ai_buy_best_shields(c, p, &budget);
          _sc_ai_buy_weapons_by_score(c, p, &budget);
+         _sc_ai_buy_shield_sappers(c, p, &budget);
          break;
 
       case SC_AI_CHOOSER:
@@ -598,6 +640,7 @@ sc_ai_result sc_ai_player_buy(const sc_config *c, sc_player *p) {
          _sc_ai_buy_contact_triggers(c, p, &budget);
          _sc_ai_buy_auto_defense(c, p, &budget);
          _sc_ai_buy_best_shields(c, p, &budget);
+         _sc_ai_buy_shield_sappers(c, p, &budget);
          _sc_ai_buy_precision_weapons(c, p, &budget);
          _sc_ai_buy_batteries(c, p, &budget);
          break;
@@ -607,6 +650,7 @@ sc_ai_result sc_ai_player_buy(const sc_config *c, sc_player *p) {
          _sc_ai_buy_contact_triggers(c, p, &budget);
          _sc_ai_buy_auto_defense(c, p, &budget);
          _sc_ai_buy_best_shields(c, p, &budget);
+         _sc_ai_buy_shield_sappers(c, p, &budget);
          _sc_ai_buy_precision_weapons(c, p, &budget);
          _sc_ai_buy_batteries(c, p, &budget);
          break;
@@ -621,6 +665,7 @@ sc_ai_result sc_ai_player_buy(const sc_config *c, sc_player *p) {
          _sc_ai_buy_auto_defense(c, p, &budget);
          _sc_ai_buy_best_shields(c, p, &budget);
          _sc_ai_buy_batteries(c, p, &budget);
+         _sc_ai_buy_shield_sappers(c, p, &budget);
          break;
 
    }
