@@ -1,16 +1,15 @@
-/* $Header: /fridge/cvs/xscorch/sai/saitraj.c,v 1.11 2001/07/08 17:36:20 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sai/saitraj.c,v 1.16 2004/02/26 06:34:54 justins Exp $ */
 /*
-   
-   xscorch - saitraj.c        Copyright(c) 2001,2000 Justin David Smith
+
+   xscorch - saitraj.c        Copyright(c) 2000-2003 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
-    
+
    AI turn code: specific functions to calculate, refine trajectory
-    
+
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,13 +21,16 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
+#include <assert.h>
+
 #include <saiint.h>              /* Main internal AI */
+
 #include <sgame/sconfig.h>       /* Need config struct */
-#include <sgame/splayer.h>       /* Need player structure */
-#include <sgame/sweapon.h>       /* Need tunneling data */
-#include <sgame/sphysics.h>      /* Need trajectories */
-#include <sgame/stankpro.h>      /* Need tank/shield radius */
 #include <sgame/sland.h>         /* We work with land/validate */
+#include <sgame/sphysics.h>      /* Need trajectories */
+#include <sgame/splayer.h>       /* Need player structure */
+#include <sgame/stankpro.h>      /* Need tank/shield radius */
+#include <sgame/sweapon.h>       /* Need tunneling data */
 
 
 
@@ -36,13 +38,20 @@
 
 
 
-static inline void _sc_ai_trajectory_compensation(const sc_config *c, const sc_player *p, const sc_player *victim, int *deltax, int *deltay) {
+static inline void _sc_ai_trajectory_compensation(const sc_config *c, const sc_player *p, 
+                                                  const sc_player *victim, int *deltax, int *deltay) {
 /* sc_ai_trajectory_compensation 
-   Compensation for shields, if offsets are allowed.  */
-   
-   if(!c->aicontrol->allowoffsets) return;
-   
-   if(victim->shield != NULL || c->aicontrol->alwaysoffset) {
+   Compensation for shields, if offset targetting is allowed.
+   The deltax, deltay values indicate the current position being
+   targeted; they may be revised to indicate a new coordinate to
+   target if offset targetting is used.  */
+
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(deltax != NULL && deltay != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
+   if(SC_AI_WILL_OFFSET(c, victim)) {
       /* deltax is always along the shortest path, so we will choose
          to compensate even _closer_, i.e. bring the target nearer
          to us in all possible cases.  If they are on a peak, this
@@ -83,7 +92,7 @@ static int _sc_ai_trajectory_angle_bounds(int deltax, int deltay, int maxdelta)
       if(vectorangle < 0) vectorangle = 180;
       delta = floor(vectorangle) - 90;
    } /* Which way are we firing? */
-   
+
    /* The AI also imposes certain max angles, as an extra sanity check. */
    if(delta > maxdelta) delta = maxdelta;
    return(delta);
@@ -93,7 +102,12 @@ static int _sc_ai_trajectory_angle_bounds(int deltax, int deltay, int maxdelta)
 
 
 static bool _sc_ai_validate_angle(const sc_config *c, sc_player *p, int angle) {
-/* sc_ai_validate_angle */
+/* sc_ai_validate_angle
+   Checks to see if there is a line of sight at least SC_AI_VALIDATE_DIST
+   pixels away from the AI tank in the specified angle.  This is to prevent
+   the AI from firing directly into an adjacent cliff, causing the AI to
+   get absorbed into its own explosion.  Returns true if the path appears
+   to be clear along the angle, and false otherwise.  This is HEURISTIC!  */
 
    double dx;
    double dy;
@@ -102,21 +116,23 @@ static bool _sc_ai_validate_angle(const sc_config *c, sc_player *p, int angle) {
    int step;
    int cx;
    int cy;
+   
+   assert(c != NULL && p != NULL);
 
    dx = SC_AI_VALIDATE_DIST * cos(angle * M_PI / 180);
    dy = SC_AI_VALIDATE_DIST * sin(angle * M_PI / 180);
-   
+
    if(fabs(dx) > fabs(dy)) {
       step = ceil(fabs(dx));
    } else {
       step = ceil(fabs(dy));
    }
-   
+
    x = sc_player_turret_x(p, angle);
    y = sc_player_turret_y(p, angle);
    dx /= step;
    dy /= step;
-   
+
    while(step > 0) {
       cx = rint(x);
       cy = rint(y);
@@ -129,9 +145,9 @@ static bool _sc_ai_validate_angle(const sc_config *c, sc_player *p, int angle) {
       y += dy;
       --step;
    }
-   
+
    return(true);
-   
+
 }
 
 
@@ -142,9 +158,9 @@ bool sc_ai_trajectory(const sc_config *c, sc_player *p, const sc_player *victim)
    only the gravity.  This function assumes we do not have a direct line of
    sight to (dx,dy); therefore, we must assume there is a tall mountain in
    the way, and find the steepest trajectory that is within our power
-   limitations.  This function returns 1 if a suitable trajectory was found,
-   and zero if the target is "unreachable".  SHOOTER's use this mechanism
-   primarily.  */
+   limitations.  This function returns true if a suitable trajectory was
+   found, and false if the target is "unreachable".  SHOOTER's use this
+   mechanism primarily.  */
 
    sc_trajectory *t; /* Trajectory (temp variable) */
    int deltax;       /* Change in X, from player's position */
@@ -153,6 +169,10 @@ bool sc_ai_trajectory(const sc_config *c, sc_player *p, const sc_player *victim)
    int power;        /* Power required to reach (dx,dy) */
    int maxdelta;     /* Maximum delta scanning angle */
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    /* Determine distance & angle between player and (dx,dy) */   
    if(!sc_land_calculate_deltas(c->land, &deltax, &deltay, p->x, p->y, victim->x, victim->y)) {
       deltax = victim->x - p->x;
@@ -172,7 +192,9 @@ bool sc_ai_trajectory(const sc_config *c, sc_player *p, const sc_player *victim)
    /* Scan for the steepest angle that gets us the trajectory we need. */
    while(abs(angle - 90) <= maxdelta) {
       if(_sc_ai_validate_angle(c, p, angle)) {
-         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_IGNORE_WIND | SC_TRAJ_IGNORE_VISC, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
+         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_IGNORE_WIND | SC_TRAJ_IGNORE_VISC, 
+                                    sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay,
+                                    angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
          if(t != NULL) {
             /* Found a suitable power level! */
             power = DIST(t->velx, t->vely) / SC_PHYSICS_VELOCITY_SCL;
@@ -180,9 +202,10 @@ bool sc_ai_trajectory(const sc_config *c, sc_player *p, const sc_player *victim)
             sc_player_set_power(c, p, power);
             sc_player_set_turret(c, p, angle);
             p->ai->victim = victim;
-            #if SC_AI_DEBUG_TRAJECTORY
-               printf("AI_trajectory:   %s, %s no-line to %d, %d with power=%d, angle=%d\n", sc_ai_name(p->aitype), p->name, deltax, deltay, power, angle);
-            #endif /* debug? */
+            if(SC_AI_DEBUG_TRAJECTORY) {
+               printf("AI_trajectory:   %s, %s no-line to %d, %d with power=%d, angle=%d\n", 
+                      sc_ai_name(p->aitype), p->name, deltax, deltay, power, angle);
+            }
             return(true);
          } /* Found a trajectory? */
       } /* Is the angle valid? */
@@ -193,7 +216,7 @@ bool sc_ai_trajectory(const sc_config *c, sc_player *p, const sc_player *victim)
 
    /* No suitable trajectory was found. */
    return(false);
-   
+
 }
 
 
@@ -214,6 +237,10 @@ bool sc_ai_trajectory_line(const sc_config *c, sc_player *p, const sc_player *vi
    int powermin;     /* Best power value found. */
    int anglemin;     /* Best angle found so far */
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    /* Determine distance & angle between player and (dx,dy) */   
    if(!sc_land_calculate_deltas(c->land, &deltax, &deltay, p->x, p->y, victim->x, victim->y)) {
       deltax = victim->x - p->x;
@@ -235,7 +262,9 @@ bool sc_ai_trajectory_line(const sc_config *c, sc_player *p, const sc_player *vi
    powermin = SC_PLAYER_MAX_POWER + 1;
    while(abs(angle - 90) <= maxdelta) {
       if(_sc_ai_validate_angle(c, p, angle)) {
-         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_IGNORE_WIND | SC_TRAJ_IGNORE_VISC | SC_TRAJ_RESTRICTIVE, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
+         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_IGNORE_WIND | SC_TRAJ_IGNORE_VISC | SC_TRAJ_RESTRICTIVE, 
+                                    sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay, 
+                                    angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
          if(t != NULL) {
             power = DIST(t->velx, t->vely) / SC_PHYSICS_VELOCITY_SCL;
             sc_traj_free(&t);
@@ -255,15 +284,16 @@ bool sc_ai_trajectory_line(const sc_config *c, sc_player *p, const sc_player *vi
       sc_player_set_power(c, p, powermin);
       sc_player_set_turret(c, p, anglemin);
       p->ai->victim = victim;
-      #if SC_AI_DEBUG_TRAJECTORY
-         printf("AI_trajectory:   %s, %s line to %d, %d with power=%d, angle=%d\n", sc_ai_name(p->aitype), p->name, deltax, deltay, powermin, anglemin);
-      #endif /* debug? */
+      if(SC_AI_DEBUG_TRAJECTORY) {
+         printf("AI_trajectory:   %s, %s line to %d, %d with power=%d, angle=%d\n", 
+                sc_ai_name(p->aitype), p->name, deltax, deltay, powermin, anglemin);
+      }
       return(true);
    }
 
    /* No suitable trajectory found */
    return(false);
-   
+
 }
 
 
@@ -289,6 +319,10 @@ bool sc_ai_trajectory_wind(const sc_config *c, sc_player *p, const sc_player *vi
    int heightmax;    /* Maximum attained height. */
    int flags;        /* Default trajectory flags */
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    /* Determine distance & angle between player and (dx,dy) */   
    if(!sc_land_calculate_deltas(c->land, &deltax, &deltay, p->x, p->y, victim->x, victim->y)) {
       deltax = victim->x - p->x;
@@ -312,7 +346,8 @@ bool sc_ai_trajectory_wind(const sc_config *c, sc_player *p, const sc_player *vi
       if(_sc_ai_validate_angle(c, p, angle)) {
          if(c->weapons->tunneling && !p->contacttriggers) flags = SC_TRAJ_TUNNELING;
          else flags = SC_TRAJ_DEFAULTS;
-         t = sc_traj_new_dest_angle(c, p, flags, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
+         t = sc_traj_new_dest_angle(c, p, flags, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), 
+                                    deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
          if(t != NULL) {
             power = DIST(t->velx, t->vely) / SC_PHYSICS_VELOCITY_SCL;
             height = sc_traj_height(t);
@@ -335,15 +370,16 @@ bool sc_ai_trajectory_wind(const sc_config *c, sc_player *p, const sc_player *vi
       sc_player_set_power(c, p, powermax);
       sc_player_set_turret(c, p, anglemax);
       p->ai->victim = victim;
-      #if SC_AI_DEBUG_TRAJECTORY
-         printf("AI_trajectory:   %s, %s no-line,wind to %d, %d with power=%d, angle=%d\n", sc_ai_name(p->aitype), p->name, deltax, deltay, powermax, anglemax);
-      #endif /* debug? */
+      if(SC_AI_DEBUG_TRAJECTORY) {
+         printf("AI_trajectory:   %s, %s no-line,wind to %d, %d with power=%d, angle=%d\n",
+                sc_ai_name(p->aitype), p->name, deltax, deltay, powermax, anglemax);
+      }
       return(true);
    } /* Found a trajectory? */
 
    /* No suitable trajectory was found. */
    return(false);
-   
+
 }
 
 
@@ -363,12 +399,16 @@ bool sc_ai_trajectory_line_wind(const sc_config *c, sc_player *p, const sc_playe
    int powermin;     /* Best power value found. */
    int anglemin;     /* Best angle found so far */
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    /* Determine distance & angle between player and (dx,dy) */   
    if(!sc_land_calculate_deltas(c->land, &deltax, &deltay, p->x, p->y, victim->x, victim->y)) {
       deltax = victim->x - p->x;
       deltay = victim->y - p->y;
    }
-   
+
    /* Compensation for shields */
    _sc_ai_trajectory_compensation(c, p, victim, &deltax, &deltay);
 
@@ -383,7 +423,9 @@ bool sc_ai_trajectory_line_wind(const sc_config *c, sc_player *p, const sc_playe
    powermin = SC_PLAYER_MAX_POWER + 1;
    while(angle <= 90 + SC_AI_SHOOTER_SIGHT_A) {
       if(_sc_ai_validate_angle(c, p, angle)) {
-         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_RESTRICTIVE, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
+         t = sc_traj_new_dest_angle(c, p, SC_TRAJ_RESTRICTIVE, 
+                                    sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), 
+                                    deltax, deltay, angle, SC_PLAYER_MAX_POWER * SC_PHYSICS_VELOCITY_SCL);
          if(t != NULL) {
             power = DIST(t->velx, t->vely) / SC_PHYSICS_VELOCITY_SCL;
             sc_traj_free(&t);
@@ -403,20 +445,23 @@ bool sc_ai_trajectory_line_wind(const sc_config *c, sc_player *p, const sc_playe
       sc_player_set_power(c, p, powermin);
       sc_player_set_turret(c, p, anglemin);
       p->ai->victim = victim;
-      #if SC_AI_DEBUG_TRAJECTORY
-         printf("AI_trajectory:   %s, %s line,wind to %d, %d with power=%d, angle=%d\n", sc_ai_name(p->aitype), p->name, deltax, deltay, powermin, anglemin);
-      #endif /* debug? */
+      if(SC_AI_DEBUG_TRAJECTORY) {
+         printf("AI_trajectory:   %s, %s line,wind to %d, %d with power=%d, angle=%d\n", 
+                sc_ai_name(p->aitype), p->name, deltax, deltay, powermin, anglemin);
+      }
       return(true);
    }
 
    /* No suitable trajectory found */
    return(false);
-   
+
 }
 
 
 
-static bool _sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_player *victim, int *newpower, int *newangle, int minangle, int maxangle, int deltaangle, int minpower, int maxpower, int deltapower, double maxdist) {
+static bool _sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_player *victim, 
+                                   int *newpower, int *newangle, int minangle, int maxangle, int deltaangle, 
+                                   int minpower, int maxpower, int deltapower, double maxdist) {
 /* _sc_ai_trajectory_scan
    Scans all possible turret angles and power levels, trying to find the
    best way to hit the intended victim.  This uses the real trajectory
@@ -435,33 +480,45 @@ static bool _sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_pl
    int tx;           /* Targetted X coordinate */
    int ty;           /* Targetted Y coordinate */
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(newpower != NULL && newangle != NULL);
+   assert(minangle <= maxangle);
+   assert(minpower <= maxpower);
+   assert(deltaangle > 0);
+   assert(deltapower > 0);
+   assert(maxdist > 0);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    /* Determine distance & angle between player and (dx,dy) */   
    if(!sc_land_calculate_deltas(c->land, &deltax, &deltay, p->x, p->y, victim->x, victim->y)) {
       deltax = victim->x - p->x;
       deltay = victim->y - p->y;
    }
-   
+
    /* Compensation for shields */
    _sc_ai_trajectory_compensation(c, p, victim, &deltax, &deltay);
-   
+
    /* Construct the desired target coordinate */
    tx = p->x + deltax;
    ty = p->y + deltay;
 
    /* Try to find a trajectory consuming the least amount of power. */   
-   #if SC_AI_DEBUG_SCAN
+   if(SC_AI_DEBUG_SCAN) {
       printf("aiscan(%d): scanning\n", p->index);
-   #endif /* SC_AI_DEBUG_SCAN */
+   }
    anglemin = 90;
    powermin = SC_PLAYER_MAX_POWER + 1;
    distmin  = 0;
    for(angle = minangle; angle <= maxangle; angle += deltaangle) {
-      #if SC_AI_DEBUG_SCAN
+      if(SC_AI_DEBUG_SCAN) {
          printf("aiscan(%d):  scanning angle %d\n", p->index, angle);
-      #endif /* SC_AI_DEBUG_SCAN */
+      }
       for(power = minpower; power < maxpower && power < SC_PLAYER_MAX_POWER; power += deltapower) {
          if(power >= SC_AI_SCAN_MIN_POWER) {
-            t = sc_traj_new_power_angle(c, p, SC_TRAJ_NO_MODIFY, sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), power * SC_PHYSICS_VELOCITY_SCL, angle);
+            t = sc_traj_new_power_angle(c, p, SC_TRAJ_NO_MODIFY, 
+                                        sc_player_turret_x(p, angle), sc_player_turret_y(p, angle), 
+                                        power * SC_PHYSICS_VELOCITY_SCL, angle);
             if(t != NULL) {
                result = sc_traj_run((sc_config *)c, t, SC_TRAJ_NO_MODIFY, NULL, NULL);
                if(SC_TRAJ_IS_IMPACT(result) && result != SC_TRAJ_IMPACT_SHIELD) {
@@ -473,19 +530,16 @@ static bool _sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_pl
                         powermin = power;
                         distmin  = dist;
                      } /* new optimal */
-                  } /* calc distance */
-                  #if SC_AI_DEBUG_SCAN
-                     else printf("    %d, %d: no deltas!\n", power, angle);
-                  #endif /* SC_AI_DEBUG_SCAN */
-               } /* trajectory */
-               #if SC_AI_DEBUG_SCAN
-                  else printf("    %d, %d: no impact!\n", power, angle);
-               #endif /* SC_AI_DEBUG_SCAN */
+                  } else if(SC_AI_DEBUG_SCAN) {
+                     printf("    %d, %d: no deltas!\n", power, angle);
+                  }
+               } else if(SC_AI_DEBUG_SCAN) {
+                  printf("    %d, %d: no impact!\n", power, angle);
+               }
                sc_traj_free(&t);
-            } /* t exists */
-            #if SC_AI_DEBUG_SCAN
-               else printf("    %d, %d: no traj!\n", power, angle);
-            #endif /* SC_AI_DEBUG_SCAN */
+            } else if(SC_AI_DEBUG_SCAN) {
+               printf("    %d, %d: no traj!\n", power, angle);
+            }
          } /* min power satisfied? */
       } /* power loop */
    } /* angle loop */
@@ -495,48 +549,54 @@ static bool _sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_pl
       /* Yes; set new trajectory and power. */
       *newpower = powermin;
       *newangle = anglemin;
-      #if SC_AI_DEBUG_SCAN
+      if(SC_AI_DEBUG_SCAN) {
          printf("aiscan(%d): done with %g at pow %d, ang %d\n", p->index, sqrt(distmin), powermin, anglemin);
-      #endif /* SC_AI_DEBUG_SCAN */
+      }
       return(true);
    }
 
    /* No suitable trajectory found */
-   #if SC_AI_DEBUG_SCAN
+   if(SC_AI_DEBUG_SCAN) {
       printf("aiscan(%d): done with %g, bad\n", p->index, sqrt(distmin));
-   #endif /* SC_AI_DEBUG_SCAN */
+   }
    return(false);
-   
+
 }
 
 
 
 bool sc_ai_trajectory_scan(const sc_config *c, sc_player *p, const sc_player *victim) {
-/* sc_ai_trajectory_scan */
+/* sc_ai_trajectory_scan
+   Scan for an optimal trajectory to the specified victim.  Returns true if
+   one was found, false otherwise.  */
 
    int power;
    int angle;
 
+   assert(c != NULL && p != NULL && victim != NULL);
+   assert(SC_PLAYER_IS_ALIVE(victim));
+   assert(p->index != victim->index);
+
    power = p->power;
    angle = p->turret;
-   #if SC_AI_DEBUG_SCAN
+   if(SC_AI_DEBUG_SCAN) {
       printf("aiscan(%d): initial configuration is  pow %d  ang %d\n", p->index, power, angle);
-   #endif /* SC_AI_DEBUG_SCAN */
+   }
 
-   if(!_sc_ai_trajectory_scan(c, p, victim, &power, &angle, angle - 20, angle + 20, 10, power - 100, power + 100, 50, 150)) return(false);
-   if(!_sc_ai_trajectory_scan(c, p, victim, &power, &angle, angle - 4, angle + 4, 1, power - 20, power + 20, 5, 50)) return(false);
+   if(!_sc_ai_trajectory_scan(c, p, victim, &power, &angle,
+                              angle - 20, angle + 20, 10,
+                              power - 100, power + 100, 50, 150)) return(false);
+   if(!_sc_ai_trajectory_scan(c, p, victim, &power, &angle,
+                              angle - 4, angle + 4, 1, 
+                              power - 20, power + 20, 5, 50)) return(false);
    if(power == p->power && angle == p->turret) return(false);
 
-   #if SC_AI_DEBUG_SCAN
+   if(SC_AI_DEBUG_SCAN) {
       printf("aiscan(%d): accepted refinement to  pow %d  ang %d\n", p->index, power, angle);
-   #endif /* SC_AI_DEBUG_SCAN */
+   }
    sc_player_set_power(c, p, power);
    sc_player_set_turret(c, p, angle);
    p->ai->victim = victim;
-
-   #if SC_AI_DEBUG_TRAJECTORY
-      printf("AI_trajectory:   %s, %s scan to dest %d, %d with power=%d, angle=%d\n", sc_ai_name(p->aitype), p->name, tx, ty, powermin, anglemin);
-   #endif /* debug? */
    return(true);
 
 }
