@@ -1,4 +1,4 @@
-/* $Header: /fridge/cvs/xscorch/sgame/saddconf.h,v 1.8 2001/12/09 01:05:53 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgame/saddconf.h,v 1.12 2004/02/26 06:34:54 justins Exp $ */
 /*
 
    xscorch - saddconf.h       Copyright(c) 2001 Jacob Luna Lundberg
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,12 +34,15 @@ struct _sc_weapon_config;
 struct _sc_accessory_config;
 
 
-/* Append to the weapon list */
-bool sc_weapon_append_file(struct _sc_weapon_config *wc, const char *filename);
+typedef enum _sc_addconf_type {
+   SC_ADDCONF_ACCESSORIES,
+   SC_ADDCONF_SCORINGS,
+   SC_ADDCONF_WEAPONS
+} sc_addconf_type;
 
 
-/* Append to the accessory list */
-bool sc_accessory_append_file(struct _sc_accessory_config *ac, const char *filename);
+/* Append to a conf list. */
+bool sc_addconf_append_file(sc_addconf_type type, const char *filename, void *container);
 
 
 #endif /* __saddconf_h_included */
