@@ -1,7 +1,7 @@
-/* $Header: /fridge/cvs/xscorch/sgame/sphoenix.c,v 1.17 2001/12/09 00:57:01 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgame/sphoenix.c,v 1.27 2004/02/26 06:34:54 justins Exp $ */
 /*
 
-   xscorch - sphoenix.c       Copyright(c) 2001,2000 Jacob Luna Lundberg
+   xscorch - sphoenix.c       Copyright(c) 2000-2003 Jacob Luna Lundberg
    jacob(at)chaos2.org        http://chaos2.org/~jacob
 
    Scorched phoenix type weapons calls
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,44 +21,42 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
+#include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <xscorch.h>
+
+#include <sphoenix.h>
+#include <scolor.h>
 #include <sconfig.h>
-#include <sweapon.h>
-#include <splayer.h>
 #include <sexplosion.h>
 #include <sinventory.h>
-#include <sphoenix.h>
-#include <sphysics.h>
-#include <swindow.h>
-#include <sshield.h>
-#include <scolor.h>
 #include <sland.h>
+#include <sphysics.h>
+#include <splayer.h>
+#include <sshield.h>
+#include <sweapon.h>
+#include <swindow.h>
+
 #include <sutil/srand.h>
-#include <math.h>
-
-#include "sexplosion.h"
 
 
-sc_phoenix_result _sc_phoenix_chain(int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
+
+static sc_phoenix_result _sc_phoenix_chain(unused int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
 /* _sc_phoenix_chain
    Split a missile with detonation on land and bounce a child if it has any */
 
    double dx, dy;
    int ph_type, ph_count;
+   sc_explosion *expl;
    sc_weapon_info *ph_i;
    sc_weapon *ph_wp;
 
-   /* No children means no phoenix, but *NOTE* that due to the
-      expense of running this function, this is not an appropriate
-      way to prevent a weapon from being a phoenix.  You should
-      leave the entire phoenix integer as 0x0000 in the weapon
-      definition if you don't want it to be a phoenix. */
+   /* What shall we reincarnate as? */
    ph_type = SC_PHOENIX_CHILD_TYPE((*wp)->weaponinfo);
    ph_count = SC_WEAPON_CHILD_COUNT(*wp);
-   /* Allow the non-phoenix or final child to detonate on its own. */
-   if(ph_type <= 0 || ph_count >= SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo))
+
+   /* Allow the final child to detonate on its own. */
+   if(ph_count >= SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo))
       return(SC_PHOENIX_NO_ACTION);
 
    /* Return subsurface weapons to the playing field. */
@@ -99,14 +96,19 @@ sc_phoenix_result _sc_phoenix_chain(int locate, const sc_config *c, sc_weapon **
    ph_wp->children = ++ph_count;
 
    /* We want the old warhead detonated now. */
-   sc_expl_add(e, sc_expl_new((*wp)->tr->curx, (*wp)->tr->cury, c->weapons->scaling * (*wp)->exp_res * ph_i->radius, ph_i->force, (*wp)->playerid, SC_EXPLOSION_NORMAL));
+   expl = sc_weapon_get_explosion(c, *wp, (*wp)->tr->curx, (*wp)->tr->cury, SC_EXPL_DEFAULT_DIR);
+   if(expl == NULL) return(SC_PHOENIX_FAILURE);
+   expl->radius = c->weapons->scaling * (*wp)->exp_res * ph_i->radius;
+   expl->force  = ph_i->force;
+   sc_expl_add(e, expl);
+
    return(SC_PHOENIX_DETONATE);
 
 }
 
 
 
-sc_phoenix_result _sc_phoenix_spider(int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
+static sc_phoenix_result _sc_phoenix_spider(unused int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
 /* _sc_phoenix_spider
    Split a missile with detonation on land and give random destinations */
 
@@ -116,14 +118,9 @@ sc_phoenix_result _sc_phoenix_spider(int locate, const sc_config *c, sc_weapon *
    double srcx;
    double srcy;
 
-   /* No children means no phoenix, but *NOTE* that due to the
-      expense of running this function, this is not an appropriate
-      way to prevent a weapon from being a phoenix.  You should
-      leave the entire phoenix integer as 0x0000 in the weapon
-      definition if you don't want it to be a phoenix. */
+   /* What shall we reincarnate as? */
    ph_type = SC_PHOENIX_CHILD_TYPE((*wp)->weaponinfo);
    ph_count = SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo);
-   if(ph_type <= 0 || ph_count <= 0) return(SC_PHOENIX_NO_ACTION);
 
    ph_i = sc_weapon_lookup(c->weapons, ph_type, SC_WEAPON_LIMIT_NONE);
    /* Random number of explosions based on no less than half of child count. */
@@ -131,15 +128,21 @@ sc_phoenix_result _sc_phoenix_spider(int locate, const sc_config *c, sc_weapon *
    leg_radius = ph_i->radius;
    leg_force = ph_i->force;
    /* Radius of blast. */
-   scatter = SC_PHOENIX_SCATTER((*wp)->weaponinfo) * (*wp)->exp_res * (*wp)->weaponinfo->radius * SC_PHOENIX_SPIDER_POWER;
+   scatter = SC_PHOENIX_SCATTER((*wp)->weaponinfo) * (*wp)->exp_res *
+             (*wp)->weaponinfo->radius * SC_PHOENIX_SPIDER_POWER;
 
-   /* First the old weapon explodes (as a plasma explosion). */
-   sc_expl_add(e, sc_expl_new((*wp)->tr->curx, (*wp)->tr->cury, c->weapons->scaling * (*wp)->exp_res * (*wp)->weaponinfo->radius, (*wp)->weaponinfo->force, (*wp)->playerid, SC_EXPLOSION_PLASMA));
+   /* First the old weapon explodes (we hope)... */
+   if(!sc_expl_add(e, sc_weapon_get_explosion(c, *wp, (*wp)->tr->curx, (*wp)->tr->cury, SC_EXPL_DEFAULT_DIR)))
+      goto failure;
 
    /* Start loop on number of spider legs (child explosions). */
    for(i = 0; i < expl_count; i++) {
       /* Create a new explosion. */
-      cure = sc_expl_new(0, 0, c->weapons->scaling * (*wp)->exp_res * leg_radius, leg_force, (*wp)->playerid, SC_EXPLOSION_SPIDER);
+      cure = sc_weapon_get_explosion(c, *wp, 0, 0, SC_EXPL_DEFAULT_DIR);
+      if(cure == NULL) goto failure;
+      cure->radius = c->weapons->scaling * (*wp)->exp_res * leg_radius;
+      cure->force  = leg_force;
+      cure->type   = SC_EXPLOSION_SPIDER;
 
       /* Pick a random x within blast radius.  The wall rules and location validation
          are performed in the low-level land manipulation and trajectory functions now.
@@ -172,15 +175,19 @@ sc_phoenix_result _sc_phoenix_spider(int locate, const sc_config *c, sc_weapon *
    /* We have detonated the initial weapon and (hopefully) some children. */
    return(SC_PHOENIX_DETONATE);
 
+   failure:
+   if(cure != NULL) sc_expl_free(&cure);
+   return(SC_PHOENIX_FAILURE);
+
 }
 
 
 
-sc_phoenix_result _sc_phoenix_scatter(int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
+static sc_phoenix_result _sc_phoenix_scatter(unused int locate, const sc_config *c, sc_weapon **wp, unused sc_explosion **e) {
 /* _sc_phoenix_scatter
    Scatter a missile without detonation */
 
-   float xscatter = 0, yscatter = 0;
+   float xscatter, yscatter;
    sc_weapon *ph_wp, *chain;
    sc_weapon_info *ph_i;
    double velx;
@@ -188,14 +195,9 @@ sc_phoenix_result _sc_phoenix_scatter(int locate, const sc_config *c, sc_weapon
    int ph_type;
    int ph_count;
 
-   /* No children means no phoenix, but *NOTE* that due to the
-      expense of running this function, this is not an appropriate
-      way to prevent a weapon from being a phoenix.  You should
-      leave the entire phoenix integer as 0x0000 in the weapon
-      definition if you don't want it to be a phoenix. */
+   /* What shall we reincarnate as? */
    ph_type = SC_PHOENIX_CHILD_TYPE((*wp)->weaponinfo);
    ph_count = SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo);
-   if(ph_type <= 0 || ph_count <= 0) return(SC_PHOENIX_NO_ACTION);
 
    /* Set some more variables. */
    ph_i = sc_weapon_lookup(c->weapons, ph_type, SC_WEAPON_LIMIT_NONE);
@@ -225,6 +227,9 @@ sc_phoenix_result _sc_phoenix_scatter(int locate, const sc_config *c, sc_weapon
       vely -= 0.5 * yscatter;
       xscatter /= ph_count;
       yscatter /= ph_count;
+   } else {
+      xscatter = 0;
+      yscatter = 0;
    }
 
    /* The weapons have monotonically differential velocities. */
@@ -245,22 +250,18 @@ sc_phoenix_result _sc_phoenix_scatter(int locate, const sc_config *c, sc_weapon
 
 
 
-sc_phoenix_result _sc_phoenix_convert(int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
+static sc_phoenix_result _sc_phoenix_convert(unused int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
 /* sc_phoenix_convert
  * Convert a weapon to a new trajectory handler */
- 
+
+   sc_phoenix_result result = SC_PHOENIX_NO_ACTION;
    int ph_type;
    int ph_count;
    int flags;
 
-   /* No children means no phoenix, but *NOTE* that due to the
-      expense of running this function, this is not an appropriate
-      way to prevent a weapon from being a phoenix.  You should
-      leave the entire phoenix integer as 0x0000 in the weapon
-      definition if you don't want it to be a phoenix. */
+   /* What shall we reincarnate as? */
    ph_type = SC_PHOENIX_CHILD_TYPE((*wp)->weaponinfo);
    ph_count = SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo);
-   if(ph_type <= 0 || ph_count <= 0) return(SC_PHOENIX_NO_ACTION);
 
    /* We'll need the weapon flags. */
    flags = SC_WEAPON_TRAJ_FLAGS(c, *wp);
@@ -268,31 +269,69 @@ sc_phoenix_result _sc_phoenix_convert(int locate, const sc_config *c, sc_weapon
    /* Maybe convert rollers. */
    if(SC_WEAPON_IS_ROLLER(*wp)) {
       /* Skip the conversion if it's already been done. */
-      if(SC_TRAJ_IS_ROLLING((*wp)->tr))
-         return(SC_PHOENIX_NO_ACTION);
+      if(SC_TRAJ_IS_ROLLING((*wp)->tr)) {
+         result = SC_PHOENIX_NO_ACTION;
+         goto out;
+      }
       /* Convert the trajectory to the new type. */
       if(!sc_trajectory_convert(c, (*wp)->tr, flags, SC_TRAJ_ROLLER)) {
          /* Manually detonate the failed conversion. */
-         sc_expl_add(e, sc_expl_new((*wp)->tr->curx, (*wp)->tr->cury, c->weapons->scaling * (*wp)->exp_res * (*wp)->weaponinfo->radius, (*wp)->weaponinfo->force, (*wp)->playerid, SC_EXPLOSION_NORMAL));
-         return(SC_PHOENIX_DETONATE);
+         if(sc_expl_add(e, sc_weapon_get_explosion(c, *wp, (*wp)->tr->curx, (*wp)->tr->cury, SC_EXPL_DEFAULT_DIR)))
+            result = SC_PHOENIX_DETONATE;
+         else
+            result = SC_PHOENIX_FAILURE;
+         goto out;
       }
+      /* We must ask the tracking code to reset itself. */
+      result = SC_PHOENIX_RESET;
    }
 
    /* Maybe convert diggers. */
    if(SC_WEAPON_IS_DIGGER(*wp)) {
       /* Skip the conversion if it's already been done. */
-      if(SC_TRAJ_IS_DIGGING((*wp)->tr))
-         return(SC_PHOENIX_NO_ACTION);
+      if(SC_TRAJ_IS_DIGGING((*wp)->tr)) {
+         result = SC_PHOENIX_NO_ACTION;
+         goto out;
+      }
       /* Convert the trajectory to the new type. */
       if(!sc_trajectory_convert(c, (*wp)->tr, flags, SC_TRAJ_DIGGER)) {
          /* Manually detonate the failed conversion. */
-         sc_expl_add(e, sc_expl_new((*wp)->tr->curx, (*wp)->tr->cury, c->weapons->scaling * (*wp)->exp_res * (*wp)->weaponinfo->radius, (*wp)->weaponinfo->force, (*wp)->playerid, SC_EXPLOSION_NORMAL));
-         return(SC_PHOENIX_DETONATE);
+         if(sc_expl_add(e, sc_weapon_get_explosion(c, *wp, (*wp)->tr->curx, (*wp)->tr->cury, SC_EXPL_DEFAULT_DIR)))
+            result = SC_PHOENIX_DETONATE;
+         else
+            result = SC_PHOENIX_FAILURE;
+         goto out;
       }
+      /* We must ask the tracking code to reset itself. */
+      result = SC_PHOENIX_RESET;
    }
 
-   /* We must ask the tracking code to reset itself. */
-   return(SC_PHOENIX_RESET);
+   out:
+   return(result);
+
+}
+
+
+
+static sc_phoenix_result _sc_phoenix_detonate(int locate, const sc_config *c, sc_weapon **wp, sc_explosion **e) {
+/* sc_phoenix_detonate
+ * Detonate a weapon in place, immediately */
+
+   double direction;
+
+   /* DETONATE is a special phoenix type; we do not require children... */
+
+   /* What direction should wedged blasts spread in? */
+   if(locate == SC_PHOENIX_AT_TANK)
+      direction = c->players[(*wp)->playerid]->turret * M_PI / 180.0;
+   else
+      direction = SC_EXPL_DEFAULT_DIR;
+
+   /* Take whatever we've got and blow it sky high! */
+   if(sc_expl_add(e, sc_weapon_get_explosion(c, *wp, (*wp)->tr->curx, (*wp)->tr->cury, direction)))
+      return(SC_PHOENIX_DETONATE);
+   else
+      return(SC_PHOENIX_FAILURE);
 
 }
 
@@ -313,23 +352,33 @@ sc_phoenix_result sc_phoenix(int locate, const sc_config *c, sc_weapon **wp, sc_
    /* Run the appropriate phoenix function(s); NOTE that although this is not
       an either/or operation, strange things may happen if you take advantage
       of this fact.  We simply haven't tested what will occur. */
+   if(SC_PHOENIX_IS_DETONATE((*wp)->weaponinfo)) {
+      result = _sc_phoenix_detonate(locate, c, wp, e);
+      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) goto out;
+   }
+   if(SC_PHOENIX_CHILD_TYPE((*wp)->weaponinfo)  <= 0 ||
+      SC_PHOENIX_CHILD_COUNT((*wp)->weaponinfo) <= 0) {
+      /* All of the remaining operations require children... */
+      goto out;
+   }
    if(SC_PHOENIX_IS_CONVERT((*wp)->weaponinfo)) {
       result = _sc_phoenix_convert(locate, c, wp, e);
-      if(result == SC_PHOENIX_RESET || result == SC_PHOENIX_FAILURE) return(result);
+      if(result == SC_PHOENIX_RESET || result == SC_PHOENIX_FAILURE) goto out;
    }
    if(SC_PHOENIX_IS_CHAIN((*wp)->weaponinfo)) {
       result = _sc_phoenix_chain(locate, c, wp, e);
-      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) return(result);
+      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) goto out;
    }
    if(SC_PHOENIX_IS_SPIDER((*wp)->weaponinfo)) {
       result = _sc_phoenix_spider(locate, c, wp, e);
-      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) return(result);
+      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) goto out;
    }
    if(SC_PHOENIX_IS_SCATTER((*wp)->weaponinfo)) {
       result = _sc_phoenix_scatter(locate, c, wp, e);
-      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) return(result);
+      if(result == SC_PHOENIX_DETONATE || result == SC_PHOENIX_FAILURE) goto out;
    }
 
+   out:
    return(result);
 
 }
@@ -341,7 +390,7 @@ typedef struct _slcache {int id; struct _slcache *next;} slcache;
 
 
 
-int _sc_phoenix_safe_verify(const sc_weapon_config *wc, const sc_weapon_info *info, slcache *cache, slcache *prev) {
+static int _sc_phoenix_safe_verify(const sc_weapon_config *wc, const sc_weapon_info *info, slcache *cache, slcache *prev) {
 /* sc_phoenix_verify
    Recursively scan the weapon's info structure and check for fatal errors. */
 
@@ -356,7 +405,8 @@ int _sc_phoenix_safe_verify(const sc_weapon_config *wc, const sc_weapon_info *in
    current.id = SC_WEAPON_TYPE(info);
    current.next = NULL;
 
-   if(SC_WEAPON_IS_PHOENIX(info)) {
+   /* DETONATE types aren't *really* phoenix weapons, per se... */
+   if(SC_WEAPON_IS_PHOENIX(info) && !SC_PHOENIX_IS_DETONATE(info)) {
       /* Find out what we're expecting as a child. */
       child = sc_weapon_lookup(wc, SC_PHOENIX_CHILD_TYPE(info), SC_WEAPON_LIMIT_NONE);
       /* A non-extant child will really throw a sabo into the machinery. */
@@ -379,7 +429,7 @@ int _sc_phoenix_safe_verify(const sc_weapon_config *wc, const sc_weapon_info *in
       if(SC_PHOENIX_IS_SCATTER(info)) count += level;
    }
 
-   /* A non-phoenix weapon is by definition acceptable.
+   /* A non-phoenix or DETONATE weapon is by definition acceptable.
       Besides, that's the stopping condition... */
 
    /* Release this level of the cache. */
@@ -420,9 +470,10 @@ static const char *_sc_phoenix_flags_bit_names[] = {
    "spider",
    "scatter",
    "convert",
+   "detonate",
    NULL
 };
-static const int _sc_phoenix_flags_bit_items[] = {
+static const unsigned int _sc_phoenix_flags_bit_items[] = {
    SC_PHOENIX_AT_RAND,
    SC_PHOENIX_AT_TANK,
    SC_PHOENIX_AT_APEX,
@@ -431,6 +482,7 @@ static const int _sc_phoenix_flags_bit_items[] = {
    SC_PHOENIX_TYPE_SPIDER,
    SC_PHOENIX_TYPE_SCATTER,
    SC_PHOENIX_TYPE_CONVERT,
+   SC_PHOENIX_TYPE_DETONATE,
    0
 };
 
@@ -444,7 +496,7 @@ const char **sc_phoenix_flags_bit_names(void) {
 
 
 
-const int *sc_phoenix_flags_bit_items(void) {
+const unsigned int *sc_phoenix_flags_bit_items(void) {
 
    return(_sc_phoenix_flags_bit_items);
 
