@@ -1,8 +1,8 @@
-/* $Header: /fridge/cvs/xscorch/sgame/splayer.c,v 1.35 2001/08/15 05:34:35 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgame/splayer.c,v 1.47 2004/02/26 06:34:54 justins Exp $ */
 /*
 
-   xscorch - splayer.c        Copyright(c) 2001,2000 Justin David Smith
-                              Copyright(c) 2001 Jacob Luna Lundberg
+   xscorch - splayer.c        Copyright(c) 2000-2003 Justin David Smith
+                              Copyright(c) 2001-2003 Jacob Luna Lundberg
    justins(at)chaos2.org      http://chaos2.org/
    jacob(at)chaos2.org        http://chaos2.org/~jacob
 
@@ -11,8 +11,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,24 +23,28 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
-#include <stdlib.h>
+#include <assert.h>
+#include <math.h>
 #include <stdio.h>
-#include <sinventory.h>
-#include <saccessory.h>
-#include <sexplosion.h>
-#include <stankpro.h>
-#include <seconomy.h>
+#include <stdlib.h>
+
 #include <splayer.h>
-#include <swindow.h>
+#include <saccessory.h>
 #include <sconfig.h>
-#include <sweapon.h>
+#include <seconomy.h>
+#include <sexplosion.h>
+#include <sinventory.h>
+#include <sland.h>
 #include <sshield.h>
 #include <sspill.h>
-#include <sland.h>
-#include <math.h>
-#include <sutil/srand.h>
-#include <sutil/sstr.h>
+#include <stankpro.h>
+#include <sweapon.h>
+#include <swindow.h>
+
 #include <snet/snet.h>
+#include <sutil/srand.h>
+
+#include <libj/jstr/libjstr.h>
 
 
 
@@ -50,7 +53,7 @@ void sc_player_init_game(const sc_config *c, sc_player *p) {
    Game initialization for this player */
 
    /* Initialise the player's currency and inventory */
-   p->money = c->economy->initialcash;
+   p->money = c->economics->initialcash;
 
    /* No wins (yet) */
    p->numwins = 0;
@@ -149,7 +152,7 @@ void sc_player_init_turn(const sc_config *c, sc_player *p) {
    } /* levitation check */
    
    /* Check tank damage; update firepower if needed */
-   life = INT_ROUND_DIV(p->life, SC_TANK_NORMAL_HARDNESS);
+   life = INT_ROUND_DIV(p->life, p->tank->hardness);
    if(p->power > life) p->power = life;
    if(p->power < 0) p->power = 0;
 
@@ -171,7 +174,7 @@ sc_player *sc_player_new(int index, const sc_tank_profile *tank) {
    if(p == NULL) return(NULL);
 
    /* Initialize player name and ID number */
-   snprintfn(p->name, sizeof(p->name), "Player %d", index + 1);
+   sbprintf(p->name, sizeof(p->name), "Player %d", index + 1);
    p->name[SC_PLAYER_NAME_LENGTH - 1] = '\0';
    p->index  = index;
    p->aitype = SC_AI_HUMAN;
@@ -275,8 +278,8 @@ void sc_player_advance_weapon(const sc_config *c, sc_player *p, int delta) {
 
    /* Cycle through the weapons */
    do {
-      if(delta > 0) info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL);
-      else info = sc_weapon_prev(c->weapons, info, SC_WEAPON_LIMIT_ALL);
+      if(delta > 0) info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_FORWARD);
+      else          info = sc_weapon_next(c->weapons, info, SC_WEAPON_LIMIT_ALL | SC_WEAPON_SCAN_REVERSE);
       if(info == NULL) {
          fprintf(stderr, "warning: player %i has no weapons (at all!), aborting search\n", p->index);
          break;
@@ -321,7 +324,8 @@ void sc_player_advance_shield(const sc_config *c, sc_player *p, int flags) {
    }
 
    /* Cycle through the shields */
-   while(info != NULL && (!sawone || info->ident != p->selshield->ident) && (!SC_ACCESSORY_IS_SHIELD(info) || info->inventories[p->index] <= 0)) {
+   while(info != NULL && (!sawone || info->ident != p->selshield->ident) && 
+         (!SC_ACCESSORY_IS_SHIELD(info) || info->inventories[p->index] <= 0)) {
       info = sc_accessory_next(c->accessories, info, SC_ACCESSORY_LIMIT_ALL);
       sawone = true;
    }
@@ -590,13 +594,15 @@ void sc_player_set_contact_triggers(const sc_config *c, sc_player *p, bool flag)
    Won't toggle it unless the player has contact triggers. */
 
    /* Sanity check */
-   if(c == NULL || p == NULL) return;
+   assert(c != NULL);
+   assert(p != NULL);
 
    /* Update the flag */
-   if(flag && (p->ac_state & SC_ACCESSORY_STATE_TRIGGER))
+   if(flag && (p->ac_state & SC_ACCESSORY_STATE_TRIGGER)) {
       p->contacttriggers = true;
-   else
+   } else {
       p->contacttriggers = false;
+   }
 
    /* Make sure player flags are transmitted over network */
    #if USE_NETWORK
@@ -610,6 +616,17 @@ void sc_player_set_contact_triggers(const sc_config *c, sc_player *p, bool flag)
 
 
 
+void sc_player_toggle_contact_triggers(const sc_config *c, sc_player *p) {
+/* sc_player_toggle_contact_triggers
+   Toggle the "use contact triggers" flag for this player.  */
+
+   assert(p != NULL);
+   sc_player_set_contact_triggers(c, p, !p->contacttriggers);
+
+}
+
+
+
 bool sc_player_use_contact_trigger(const sc_config *c, sc_player *p) {
 /* sc_player_use_contact_trigger
    Try to use a contact trigger.  If successful, return true. */
@@ -728,7 +745,7 @@ void sc_player_inc_wins(sc_config *c, sc_player *p) {
 /* sc_player_inc_wins */
 
    ++p->numwins;
-   p->money += c->economy->survivalbonus;
+   p->money += c->economics->survivalbonus;
 
 }
 
@@ -738,7 +755,7 @@ void sc_player_died(sc_config *c, sc_player *p) {
 /* sc_player_died */
 
    p->dead = true;
-   p->money -= c->economy->deathloss;
+   p->money -= c->economics->deathloss;
 
 }
 
@@ -756,6 +773,9 @@ static bool _sc_player_drop(sc_config *c, sc_player *p, int falldist) {
    if(SC_CONFIG_NO_ANIM(c)) dropdist = c->fieldheight;
    else dropdist = falldist;
 
+   /* Note: this function needs to use the dead flag; if the player is
+      technically lifeless but the tank has not been removed from the
+      playfield yet, then the tank should still take a plunge.  */
    if(!p->dead) {
       /* Get maximum height of land around and just below the tank */
       falldist = 0;
@@ -806,6 +826,8 @@ bool sc_player_drop_all(sc_config *c) {
    needsrecurse = 0;
    i = c->numplayers - 1;
    while(i >= 0) {
+      /* Make sure _sc_player_drop is FIRST, so short-circuiting does
+         not kill the function call (which has side effects).  */
       needsrecurse = _sc_player_drop(c, c->players[i], SC_TANK_MAX_DROP_PER_CYCLE) || needsrecurse;
       --i;
    }
@@ -822,7 +844,7 @@ static void _sc_player_damage(sc_config *c, sc_player *p, const sc_explosion *e)
    int damage;
 
    /* If player is already dead, then there's nothing to do */
-   if(p->dead || p->life <= 0) return;
+   if(!SC_PLAYER_IS_ALIVE(p)) return;
 
    /* How much damage was taken? */
    damage = sc_expl_damage_at_point(c->land, e, p->x, p->y);
@@ -836,28 +858,30 @@ static void _sc_player_damage(sc_config *c, sc_player *p, const sc_explosion *e)
    p->life -= damage * SC_TANK_NORMAL_HARDNESS;
 
    /* Revive shields if necessary (and if auto-defense enabled) */
-   sc_player_activate_auto_shield(c, p);
+   if(SC_PLAYER_IS_ALIVE(p)) {
+      sc_player_activate_auto_shield(c, p);
+   }
 
    /* Tank took a direct or partial hit.  Damn. */
-   p->money -= c->economy->damageloss;
-   if(p->life <= 0) {
+   p->money -= c->economics->damageloss;
+   if(!SC_PLAYER_IS_ALIVE(p)) {
       p->life = 0;  /* We died */
       p->killedby = e->playerid;
       /* Who killed us? */
       if(e->playerid == p->index) {
          /* It was a suicide */
          ++c->players[e->playerid]->suicides;
-         p->money -= c->economy->suicideloss;
+         p->money -= c->economics->suicideloss;
       } else {
          /* Give opponent some points for killing us */
          ++c->players[e->playerid]->kills;
-         c->players[e->playerid]->money += c->economy->killbonus;
+         c->players[e->playerid]->money += c->economics->killbonus;
       }
    }
 
    if(e->playerid != p->index) {
       /* Give points to other player for damaging us */
-      c->players[e->playerid]->money += c->economy->damagebonus;
+      c->players[e->playerid]->money += c->economics->damagebonus;
    } /* Make sure not self */
 
 }
@@ -1081,24 +1105,28 @@ void sc_player_death(const sc_config *c, const sc_player *p, sc_explosion **e) {
 
    sc_explosion *expl;
 
-   if(c == NULL || p == NULL || e == NULL) return;
+   assert(c != NULL && p != NULL && e != NULL);
 
    switch(game_lrand(10)) {
 
       case 0:
          /* Just say it with napalm ... */
-         expl = sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_NAPALM_RADIUS, SC_WEAPON_NAPALM_FORCE, p->killedby, SC_EXPLOSION_NAPALM);
+         expl = sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_NAPALM_RADIUS,
+                            SC_WEAPON_NAPALM_FORCE, p->killedby, SC_EXPLOSION_NAPALM);
          expl->data = sc_spill_new(c, c->land, SC_WEAPON_NAPALM_LIQUID, expl->centerx, expl->centery);
          sc_expl_add(e, expl);
          break;
 
       default:
          /* Just the usual, 3-stage detonation */
-         sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_SMALL_EXPLOSION, SC_WEAPON_SMALL_FORCE, p->killedby, SC_EXPLOSION_NORMAL));
+         sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_SMALL_EXPLOSION, 
+                                    SC_WEAPON_SMALL_FORCE, p->killedby, SC_EXPLOSION_NORMAL));
          if(game_drand() < 0.5) {
-            sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_MEDIUM_EXPLOSION, SC_WEAPON_MEDIUM_FORCE, p->killedby, SC_EXPLOSION_NORMAL));
+            sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_MEDIUM_EXPLOSION, 
+                                       SC_WEAPON_MEDIUM_FORCE, p->killedby, SC_EXPLOSION_NORMAL));
             if(game_drand() < 0.1) {
-               sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_LARGE_EXPLOSION, SC_WEAPON_LARGE_FORCE, p->killedby, SC_EXPLOSION_PLASMA));
+               sc_expl_add(e, sc_expl_new(p->x, p->y, c->weapons->scaling * SC_WEAPON_LARGE_EXPLOSION, 
+                                          SC_WEAPON_LARGE_FORCE, p->killedby, SC_EXPLOSION_PLASMA));
             } /* Do third stage? */
          } /* Do second stage? */
          break;
@@ -1116,7 +1144,7 @@ bool sc_player_passable(const sc_config *c, const sc_player *p, int x, int y) {
    this we actually check the tank profile to make sure the land is
    passable everywhere where the tank profile is opaque. */
 
-   const char *data;
+   const unsigned char *data;
    int radius;
    int cx;
    int cy;
@@ -1146,7 +1174,7 @@ bool sc_player_would_impact(const sc_config *c, const sc_player *p, int x, int y
    coordinate is a virtual coordinate; returns true if an impact would
    occur. */
 
-   const char *data;
+   const unsigned char *data;
    int radius;
    int dx;
    int dy;
@@ -1255,7 +1283,7 @@ const char *sc_player_talk(const sc_config *c, const sc_player *p) {
       case SC_CONFIG_TALK_COMPUTERS:
          if(p->aitype == SC_AI_HUMAN) break;
       default:
-         if(game_lrand(100) < c->options.talkprob) {
+         if(game_lrand(100) < (dword)c->options.talkprob) {
             return(_sc_player_messages[game_lrand(SC_PLAYER_NUM_MESSAGES)]);
          }
    }
@@ -1274,7 +1302,7 @@ const char *sc_player_death_talk(const sc_config *c, const sc_player *p) {
       case SC_CONFIG_TALK_COMPUTERS:
          if(p->aitype == SC_AI_HUMAN) break;
       default:
-         if(game_lrand(100) < c->options.talkprob) {
+         if(game_lrand(100) < (dword)c->options.talkprob) {
             return(_sc_player_death_messages[game_lrand(SC_PLAYER_NUM_DEATH_MESSAGES)]);
          }
    }
