@@ -1,42 +1,42 @@
-/* $Header: /fridge/cvs/xscorch/sgame/sstate.c,v 1.12 2001/07/31 02:29:31 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgame/sstate.c,v 1.23 2004/02/26 06:34:54 justins Exp $ */
 /*
-   
-   xscorch - sstate.c         Copyright(c) 2001,2000 Justin David Smith
+
+   xscorch - sstate.c         Copyright(c) 2000-2004 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
 
    Scorched game state machine
-    
 
-   This program is free software; you can redistribute it and/or modify 
-   it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful, 
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation, version 2 of the License ONLY.
+
+   This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation, 
+   along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
-#include <sgame.h>         /* Game state header */
 #include <sstate.h>        /* Game state header */
-#include <sphysics.h>      /* Need to do Wind updates */
-#include <seconomy.h>      /* Need to do interest update */
-#include <spreround.h>     /* For Auto Defense and The Lottery */
-#include <swindow.h>       /* We do much drawing from state machine */
 #include <sconfig.h>       /* Config is dereferenced frequently */
-#include <splayer.h>       /* Dereferenced in death check, etc. */
-#include <sweapon.h>       /* We construct weapon chains here */
-#include <strack.h>        /* Need for tracking, return codes */
-#include <ssound/ssound.h> /* Sound hooks are present, here  */
+#include <seconomy.h>      /* Need to do interest update */
+#include <sgame.h>         /* Game state header */
 #include <sland.h>         /* Need to drop the land after deton */
-#include <sutil/srand.h>   /* Need a random number generator */
+#include <sphysics.h>      /* Need to do Wind updates */
+#include <splayer.h>       /* Dereferenced in death check, etc. */
+#include <spreround.h>     /* For Auto Defense and The Lottery */
+#include <strack.h>        /* Need for tracking, return codes */
+#include <sweapon.h>       /* We construct weapon chains here */
+#include <swindow.h>       /* We do much drawing from state machine */
+
 #include <sai/sai.h>       /* We are responsible for running AI */
-#include <snet/snet.h>
+#include <snet/snet.h>     /* This is a comment about snet.h */
+#include <ssound/ssound.h> /* Sound hooks are present, here  */
+#include <sutil/srand.h>   /* Need a random number generator */
 
 
 
@@ -45,10 +45,10 @@ static void _sc_state_prelude_begin(sc_config *c, sc_game *g) {
    /* Start sound, if applicable */
    g->musicid = SC_MUSIC_PRELUDE;
    sc_sound_start(c->sound, g->musicid);
-            
+
    /* Setup screen/status */
    sc_status_message(c->window, "Welcome to XScorch " VERSION "!");
-            
+
    /* Activate the main menu and wait */
    sc_window_main_menu(c->window);
    sc_game_set_state(g, SC_STATE_PRELUDE_IDLE, SC_DELAY_PROMPT);
@@ -58,7 +58,7 @@ static void _sc_state_prelude_begin(sc_config *c, sc_game *g) {
 
 
 static void _sc_state_game_end(sc_config *c, sc_game *g) {
-   
+
    /* Start end-game sound */
    g->musicid = SC_MUSIC_ENDGAME;
    sc_sound_start(c->sound, g->musicid);
@@ -83,7 +83,7 @@ static void _sc_state_inventory_begin(sc_config *c, sc_game *g) {
       if(c->client != NULL) sc_status_message(c->window, "Waiting for other players to begin round ...");
       sc_net_client_sync(c->client, SC_CONN_SYNC_INV, c->server != NULL);
    #endif /* Network? */
-   sc_game_set_state_asap(g, SC_STATE_INVENTORY_PL_BEGIN);
+   sc_game_set_state_now(c, g, SC_STATE_INVENTORY_PL_BEGIN);
 
 }
 
@@ -111,7 +111,7 @@ static void _sc_state_inventory_pl_done(sc_config *c, sc_game *g) {
    #if USE_NETWORK
       sc_net_client_send_inventory(c, c->client, g->curplayer);
    #endif /* Networking? */
-   
+
    ++g->curplayer;
    if(g->curplayer >= c->numplayers) {
       g->curplayer = 0;
@@ -119,7 +119,7 @@ static void _sc_state_inventory_pl_done(sc_config *c, sc_game *g) {
    } else {
       sc_game_set_state_now(c, g, SC_STATE_INVENTORY_PL_BEGIN);
    }
-   
+
 }
 
 
@@ -191,7 +191,7 @@ static void _sc_state_lottery_display(sc_config *c, sc_game *g) {
 static void _sc_state_round_end(sc_config *c, sc_game *g) {
 
    sc_game_set_victor(c);
-   sc_economy_interest(c, c->economy);
+   sc_economy_interest(c, c->economics);
 
    if(c->curround + 1 >= c->numrounds) {
       sc_game_set_state_now(c, g, SC_STATE_GAME_END);
@@ -320,9 +320,10 @@ static void _sc_state_run_talk(sc_config *c, sc_game *g) {
       if(!SC_CONFIG_GFX_FAST(c) && msg != NULL) {
          sc_status_player_message(c->window, p, msg);
          sc_game_set_state(g, nextstate, SC_DELAY_TALK);
+         return;
       }
    }
-   
+
    sc_game_set_state_now(c, g, nextstate);
 
 }
@@ -334,9 +335,14 @@ static void _sc_state_run_creation(sc_config *c, sc_game *g) {
    g->activity = false;
    sc_status_message(c->window, "");
    sc_game_sync_timeout(g);
-   sc_weapon_create_all(c, &(g->expl_init));
-   sc_game_set_state_now(c, g, SC_STATE_RUN_WEAPONS);
-   
+
+   /* If sc_weapon_create_all returns true, we have explosions to run. */
+   if(sc_weapon_create_all(c, &(g->expl_init))) {
+      sc_game_set_state_now(c, g, SC_STATE_RUN_EXPLOSION);
+   } else {
+      sc_game_set_state_now(c, g, SC_STATE_RUN_WEAPONS);
+   }
+
 }
 
 
@@ -370,7 +376,7 @@ static void _sc_state_run_weapons(sc_config *c, sc_game *g) {
          break;
 
       default:
-         /* Oops */
+         /* Oops */;
    }
 
 }
@@ -416,7 +422,7 @@ static bool _sc_state_run_clear_expl(sc_config *c, sc_game *g, sc_explosion *e)
       clear is called, as damage_all might need information that is
       deallocated on annihilate_clear().  */
    sc_player_damage_all(c, e);
-   
+
    if(!sc_expl_annihilate_clear(c, e)) {
       /* This explosion was cleared quickly */
       sc_game_expl_queue_item_move(&g->expl_done, &g->expl_clear_init, e);
@@ -464,22 +470,22 @@ static bool _sc_state_run_explosion_simult(sc_config *c, sc_game *g) {
 
    /* Process ``init'' queue */
    result = sc_game_expl_queue_do_runnable(c, g, g->expl_init, _sc_state_run_draw_expl);
-   
+
    /* Process ``draw'' queue */
    result = sc_game_expl_queue_do_runnable(c, g, g->expl_draw, _sc_state_run_draw_expl_cont) | result;
-   
+
    /* Process ``clear_init'' queue */
    result = sc_game_expl_queue_do_runnable(c, g, g->expl_clear_init, _sc_state_run_clear_expl) | result;
-   
+
    /* Process ``clear'' queue */
    result = sc_game_expl_queue_do_runnable(c, g, g->expl_clear, _sc_state_run_clear_expl_cont) | result;
-   
+
    /* Process ``done'' queue */
    result = sc_game_expl_queue_do_runnable(c, g, g->expl_done, _sc_state_run_land_fall) | result;
-   
+
    /* Were any actions taken? */
    return(result);
-   
+
 }
 
 
@@ -488,7 +494,7 @@ static bool _sc_state_run_explosion_nosimult(sc_config *c, sc_game *g) {
 
    /* Check ``done'' queue */
    if(g->expl_done != NULL) return(sc_game_expl_queue_run_first(c, g, g->expl_done, _sc_state_run_land_fall));
-   
+
    /* Check ``clear'' queue */
    if(g->expl_clear != NULL) return(sc_game_expl_queue_run_first(c, g, g->expl_clear, _sc_state_run_clear_expl_cont));
 
@@ -500,7 +506,7 @@ static bool _sc_state_run_explosion_nosimult(sc_config *c, sc_game *g) {
 
    /* Check ``init'' queue */
    if(g->expl_init != NULL) return(sc_game_expl_queue_run_first(c, g, g->expl_init, _sc_state_run_draw_expl));
-   
+
    /* No actions taken */
    return(false);
 
@@ -518,7 +524,7 @@ static void _sc_state_run_explosion(sc_config *c, sc_game *g) {
 
    do {
       #if SC_STATE_DEBUG
-      printf("run_explosion:  queues init %d  draw %d  clear_init %d  clear %d  done %d\n", 
+      printf("run_explosion:  queues init %d  draw %d  clear_init %d  clear %d  done %d\n",
              sc_expl_count(g->expl_init),
              sc_expl_count(g->expl_draw),
              sc_expl_count(g->expl_clear_init),
@@ -537,11 +543,11 @@ static void _sc_state_run_explosion(sc_config *c, sc_game *g) {
          result = _sc_state_run_explosion_nosimult(c, g);
       }
    } while(result);
-   
+
    #if SC_STATE_DEBUG
    printf("run_explosion:  done\n");
    #endif /* SC_STATE_DEBUG */
-   
+
    /* Update the counters */
    sc_game_expl_queue_update(g->expl_init);
    sc_game_expl_queue_update(g->expl_draw);
@@ -643,9 +649,9 @@ static void _sc_state_run_end(sc_config *c, sc_game *g) {
 void sc_state_run(sc_config *c, sc_game *g) {
 
    struct timeval curtime;
-   
+
    /* Check network activity */
-   #if USE_NETWORK 
+   #if USE_NETWORK
       sc_net_client_run(c, c->client);
       sc_net_server_run(c, c->server);
       if(sc_net_client_death(&c->client)) {
@@ -657,16 +663,20 @@ void sc_state_run(sc_config *c, sc_game *g) {
          return;
       }
    #endif /* Network activity? */
-   
+
    sc_game_time(&curtime);
-   if(curtime.tv_sec > g->timeout.tv_sec || 
+   if(curtime.tv_sec > g->timeout.tv_sec ||
      (curtime.tv_sec == g->timeout.tv_sec && curtime.tv_usec >= g->timeout.tv_usec)) {
 
+      #if SC_STATE_TIMER_DEBUG
+         printf("...  state %8x accepted %ld.%06ld\n", g->state, curtime.tv_sec, curtime.tv_usec);
+      #endif /* SC_STATE_TIMER_DEBUG */
+
       #if SC_STATE_DEBUG
-         printf("State %8x    ", g->state); 
+         printf("State %8x    ", g->state);
          SC_PROFILE_BEGIN("game_run")
       #endif /* SC_STATE_DEBUG */
-      
+
       switch(g->state) {
 
 
@@ -674,9 +684,9 @@ void sc_state_run(sc_config *c, sc_game *g) {
          /***  PRELUDE  ***/
 
 
-         /* PRELUDE_BEGIN  
+         /* PRELUDE_BEGIN
                -> PRELUDE_IDLE
-            Initialise for a new game; display main menu. 
+            Initialise for a new game; display main menu.
          */
          case SC_STATE_PRELUDE_BEGIN:
          case SC_STATE_GAME_END_DONE:
@@ -688,7 +698,7 @@ void sc_state_run(sc_config *c, sc_game *g) {
          /***  GAME  ***/
 
 
-         /* GAME_BEGIN     
+         /* GAME_BEGIN
                -> INVENTORY_BEGIN   (immediate)
             When user first presses Enter on the main menu.  Initialize
             configuration: Initialize config, player game data, and just
@@ -718,8 +728,8 @@ void sc_state_run(sc_config *c, sc_game *g) {
 
 
          /***  INVENTORY   ***/
-         
-         
+
+
          /* INVENTORY_BEGIN
                -> INVENTORY_PL_BEGIN   (immediate)
             Begin inventory; prepare to take inventory for one player.
@@ -770,8 +780,8 @@ void sc_state_run(sc_config *c, sc_game *g) {
 
 
          /***  ROUND    ***/
-         
-         
+
+
          /* ROUND_BEGIN
                -> TURN_BEGIN     (immediate)
             Begin a new round:  Inventory has been taken; generate the land,
@@ -780,25 +790,25 @@ void sc_state_run(sc_config *c, sc_game *g) {
          case SC_STATE_ROUND_BEGIN:
             _sc_state_round_begin(c, g);
             break;
-            
-            
+
+
          /* ROUND_END
                -> ROUND_END_IDLE (end of round, will loop)
-               t> GAME_END       (end of all rounds) 
+               t> GAME_END       (end of all rounds)
             Round has ended; advance round, and go to inventory or final
-            winnings (latter occurs only if all rounds have been played). 
+            winnings (latter occurs only if all rounds have been played).
             In former, we identify the victor, do economics and final
             winning calculations, and then notify the user who won.
          */
          case SC_STATE_ROUND_END:
             _sc_state_round_end(c, g);
             break;
-            
+
 
 
          /***  TURNS    ***/
-         
-         
+
+
          /* TURN_BEGIN
                -> TURN_PL_BEGIN  (normal progression)
                t> ROUND_END      (no one/1 person alive)
@@ -827,17 +837,17 @@ void sc_state_run(sc_config *c, sc_game *g) {
          case SC_STATE_TURN_PL_BEGIN:
             _sc_state_turn_pl_begin(c, g);
             break;
-            
-            
+
+
          /* TURN_PL_DONE
-         
+
             ==Sequential Mode==
                -> RUN_TALK
             This player has given orders; execute those orders if the player
             is living.  If the player is dead, RUN_TALK will fall through
             and proceed to the next player.  The player's weapon is armed
             here.
-               
+
             ==Synchronous Mode==
                -> TURN_PL_NEXT
             This player has given orders; take orders from the next player.
@@ -850,11 +860,11 @@ void sc_state_run(sc_config *c, sc_game *g) {
 
 
          /* TURN_PL_NEXT
-            
+
             ==Sequential Mode==
                -> TURN_PL_BEGIN     (immediate)
                t> TURN_END          (terminal, immediate)
-               
+
             ==Synchronous Mode==
                -> TURN_PL_BEGIN     (immediate)
                t> RUN_TALK          (terminal, immediate)
@@ -869,36 +879,39 @@ void sc_state_run(sc_config *c, sc_game *g) {
 
 
          /* RUN_TALK
-            
+
             ==Sequential Mode==
                -> RUN_CREATION      (might be 2s delay)
             This state DOES NOT LOOP.  If this player is not dead and armed,
             then we might allow them to speak (2s delay) or might launch
             their weapon now (immediate).
-               
+
             ==Synchronous Mode==
                -> RUN_TALK          (might delay, loops)
                t> RUN_CREATION      (out of players)
             This state loops through all players.  If the player is not dead
             and is armed, they may speak at this time (2s delay) or might
             not (loop is immediate).  Once complete, this will yield to the
-            first weapons state.
-         */            
+            weapon creation state.
+         */
          case SC_STATE_RUN_TALK:
             _sc_state_run_talk(c, g);
             break;
 
 
          /* RUN_CREATION
-               -> RUN_WEAPONS       (immediate)
-            Create all player weapons.
+               -> RUN_WEAPONS          (immediate; no explosions yet)
+               -> RUN_EXPLOSION        (immediate; explosions waiting)
+            Create all player weapons.  If the weapon creation created
+            an explosion immediately, then process the explosion before
+            sending any weapons into flight.
          */
          case SC_STATE_RUN_CREATION:
             _sc_state_run_creation(c, g);
             break;
 
 
-         /* RUN_WEAPONS (TEMP OBSOLETE DOC)
+         /* RUN_WEAPONS (TEMP OBS DOC)
                -> RUN_WEAPONS          (weapons still flying)
                -> RUN_EXPLOSION        (might be flying; detonation)
                t> RUN_END              (terminal case; no weapons)
@@ -909,7 +922,7 @@ void sc_state_run(sc_config *c, sc_game *g) {
             break;
 
 
-         /* RUN_EXPLOSION (TEMP OBSOLETE DOC)
+         /* RUN_EXPLOSION (TEMP OBS DOC)
                -> RUN_EXPLOSION        (this explosion chain not done; wait)
                t> RUN_PLAYER_DROP      (immediate)
             Run an explosion in progress.  The weapon explosion might opt to
@@ -923,7 +936,7 @@ void sc_state_run(sc_config *c, sc_game *g) {
             break;
 
 
-         /* RUN_PLAYER_DROP (TEMP OBSOLETE DOC)
+         /* RUN_PLAYER_DROP (TEMP OBS DOC)
                -> RUN_PLAYER_DROP   (loop)
                t> RUN_PLAYER_DEATH  (terminal)
          */
@@ -932,7 +945,7 @@ void sc_state_run(sc_config *c, sc_game *g) {
             break;
 
 
-         /* RUN_PLAYER_DEATH (TEMP OBSOLETE DOC)
+         /* RUN_PLAYER_DEATH (TEMP OBS DOC)
                -> RUN_PLAYER_DEATH
                t> RUN_END
             Check if any players just died.
@@ -940,13 +953,13 @@ void sc_state_run(sc_config *c, sc_game *g) {
          case SC_STATE_RUN_PLAYER_DEATH:
             _sc_state_run_player_death(c, g);
             break;
-            
-            
+
+
          /* RUN_END
-            
+
             ==Sequential Mode==
                -> TURN_PL_NEXT      (immediate)
-               
+
             ==Synchronous Mode==
                -> TURN_END          (immediate)
          */
@@ -967,7 +980,7 @@ void sc_state_run(sc_config *c, sc_game *g) {
       #if SC_STATE_DEBUG
          SC_PROFILE_END
       #endif /* SC_STATE_DEBUG */
- 
+
    }
 
 }
