@@ -1,16 +1,17 @@
-/* $Header: /fridge/cvs/xscorch/sgtk/sautodef-gtk.c,v 1.21 2001/12/10 08:28:43 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgtk/sautodef-gtk.c,v 1.33 2004/02/26 06:34:55 justins Exp $ */
 /*
 
-   xscorch - sautodef-gtk.c   Copyright(c) 2001 Jacob Luna Lundberg
+   xscorch - sautodef-gtk.c   Copyright(c) 2001-2003 Jacob Luna Lundberg
+                              Copyright(c) 2003 Justin David Smith
    jacob(at)chaos2.org        http://chaos2.org/~jacob
+   justins(at)chaos2.org      http://chaos2.org/
 
    Auto Defense settings screen
 
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,26 +23,28 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
-#include <stdlib.h>
-#include <stdio.h>
 #include <math.h>
-#include <xscorch.h>
-#include <sactoggle.h>
-#include <sdisplay.h>
+#include <stdio.h>
+#include <stdlib.h>
+
+#include <sgtk.h>
 #include <sactiveconsole.h>
-#include <gdk/gdkkeysyms.h>
-#include <swindow-gtk.h>
+#include <sactoggle.h>
 #include <scolor-gtk.h>
+#include <sdisplay.h>
+#include <swindow-gtk.h>
+
 #include <sai/sai.h>
 #include <sgame/saccessory.h>
-#include <sgame/spreround.h>
 #include <sgame/sconfig.h>
-#include <sgame/splayer.h>
-#include <sgame/sweapon.h>
-#include <sgame/sstate.h>
 #include <sgame/sgame.h>
-#include <sutil/sstr.h>
-#include <sgtk.h>
+#include <sgame/splayer.h>
+#include <sgame/spreround.h>
+#include <sgame/sstate.h>
+#include <sgame/sweapon.h>
+
+#include <gdk/gdkkeysyms.h>
+#include <libj/jstr/libjstr.h>
 
 
 
@@ -153,13 +156,17 @@ inline void _sc_window_auto_def_engage(sc_auto_def_gtk *adg) {
 
 
 
-static gboolean _sc_window_auto_def_close_gtk(GtkWidget *widget, ScActiveConsoleSpot *spot, sc_auto_def_gtk *adg) {
+static gboolean _sc_window_auto_def_close_gtk(unused GtkWidget *widget,
+                                              unused ScActiveConsoleSpot *spot,
+                                              sc_auto_def_gtk *adg) {
 /* _sc_window_auto_def_close_gtk
    Enact the auto defense settings and restart the auto defense loop. */
 
+   sc_window_gtk *w = adg->w;
+
    if(!adg->ready) return(FALSE);
 
-   #if SC_GTK_DEBUG_GTK
+   #if SC_GTK_DEBUG_GTK && __debugging_macros
       SC_DEBUG_ENTER("accepting autodef close%s", "");
    #endif /* debug */
 
@@ -167,9 +174,9 @@ static gboolean _sc_window_auto_def_close_gtk(GtkWidget *widget, ScActiveConsole
    _sc_window_auto_def_engage(adg);
 
    /* Destroy the display and restart the auto defense player loop. */
-   sc_display_console_detach_all(SC_DISPLAY(adg->w->screen));
-   sc_game_set_state_now(adg->w->c, adg->w->c->game, SC_STATE_AUTO_DEFENSE_LOOP);
-   sc_status_message((sc_window *)adg->w, "");
+   sc_display_console_detach_all(SC_DISPLAY(w->screen));
+   sc_game_set_state_now(w->c, w->c->game, SC_STATE_AUTO_DEFENSE_LOOP);
+   sc_status_message((sc_window *)w, "");
    _sc_auto_def_gtk_free(&adg);
    return(TRUE);
 
@@ -177,13 +184,17 @@ static gboolean _sc_window_auto_def_close_gtk(GtkWidget *widget, ScActiveConsole
 
 
 
-static gboolean _sc_window_auto_def_key_gtk(GtkWidget *widget, ScActiveConsoleSpot *spot, GdkEventKey *key, sc_auto_def_gtk *adg) {
+static gboolean _sc_window_auto_def_key_gtk(unused GtkWidget *widget,
+                                            unused ScActiveConsoleSpot *spot,
+                                            GdkEventKey *key, sc_auto_def_gtk *adg) {
 /* _sc_window_auto_def_key_gtk
    Enact the auto defense settings and restart the auto defense loop. */
 
+   sc_window_gtk *w = adg->w;
+
    if(!adg->ready) return(FALSE);
 
-   #if SC_GTK_DEBUG_GTK
+   #if SC_GTK_DEBUG_GTK && __debugging_macros
       SC_DEBUG_ENTER("got autodef cancel (esc)%s", "");
    #endif /* debug */
 
@@ -194,8 +205,8 @@ static gboolean _sc_window_auto_def_key_gtk(GtkWidget *widget, ScActiveConsoleSp
          _sc_window_auto_def_engage(adg);
       case GDK_Escape:
          /* Destroy the display and restart the auto defense player loop. */
-         sc_display_console_detach_all(SC_DISPLAY(adg->w->screen));
-         sc_game_set_state_now(adg->w->c, adg->w->c->game, SC_STATE_AUTO_DEFENSE_LOOP);
+         sc_display_console_detach_all(SC_DISPLAY(w->screen));
+         sc_game_set_state_now(w->c, w->c->game, SC_STATE_AUTO_DEFENSE_LOOP);
          _sc_auto_def_gtk_free(&adg);
          return(TRUE);
    }
@@ -205,7 +216,9 @@ static gboolean _sc_window_auto_def_key_gtk(GtkWidget *widget, ScActiveConsoleSp
 
 
 
-static gboolean _sc_autodef_guidance_enter_gtk(ScActiveConsole *cons, ScActiveConsoleSpot *spot, sc_auto_def_gtk *adg) {
+static gboolean _sc_autodef_guidance_enter_gtk(unused ScActiveConsole *cons,
+                                               ScActiveConsoleSpot *spot,
+                                               sc_auto_def_gtk *adg) {
 /* _sc_autodef_guidance_enter_gtk
    Move the selection to a new item in the list. */
 
@@ -213,7 +226,7 @@ static gboolean _sc_autodef_guidance_enter_gtk(ScActiveConsole *cons, ScActiveCo
 
    if(!adg->ready) return(FALSE);
 
-   #if SC_GTK_DEBUG_GTK
+   #if SC_GTK_DEBUG_GTK && __debugging_macros
       SC_DEBUG_ENTER("got autodef guidance entry%s", "");
    #endif /* debug */
 
@@ -233,7 +246,9 @@ static gboolean _sc_autodef_guidance_enter_gtk(ScActiveConsole *cons, ScActiveCo
 
 
 
-static gboolean _sc_autodef_shield_enter_gtk(ScActiveConsole *cons, ScActiveConsoleSpot *spot, sc_auto_def_gtk *adg) {
+static gboolean _sc_autodef_shield_enter_gtk(unused ScActiveConsole *cons,
+                                             ScActiveConsoleSpot *spot,
+                                             sc_auto_def_gtk *adg) {
 /* _sc_autodef_shield_enter_gtk
    Move the selection to a new item in the list. */
 
@@ -241,7 +256,7 @@ static gboolean _sc_autodef_shield_enter_gtk(ScActiveConsole *cons, ScActiveCons
 
    if(!adg->ready) return(FALSE);
 
-   #if SC_GTK_DEBUG_GTK
+   #if SC_GTK_DEBUG_GTK && __debugging_macros
       SC_DEBUG_ENTER("got autodef shield entry%s", "");
    #endif /* debug */
 
@@ -268,7 +283,8 @@ void _sc_window_auto_defense_guidance_init(sc_auto_def_gtk *adg) {
 
    int count;
    char buf[SC_GTK_STRING_BUFFER];
-   sc_accessory_config *ac = adg->w->c->accessories;
+   sc_window_gtk *w = adg->w;
+   sc_accessory_config *ac = w->c->accessories;
    sc_accessory_info *info;
 
    /* Make a title on the main window. */
@@ -294,11 +310,11 @@ void _sc_window_auto_defense_guidance_init(sc_auto_def_gtk *adg) {
    adg->set_guidance.items = (int *)realloc(adg->set_guidance.items, adg->set_guidance.length * sizeof(int));
 
    /* Create the guidance system window. */
-   adg->console_guidance = sc_active_console_new(9, 9, 32, 3, CONSOLE_NORMAL);
+   adg->console_guidance = sc_window_active_console_new(w, 9, 9, 32, 3, CONSOLE_NORMAL);
    sc_console_buffer_size(SC_CONSOLE(adg->console_guidance), 32, adg->set_guidance.length + 1);
 
    /* Draw the ``nothing selected'' item and hook it up. */
-   snprintfn(buf, sizeof(buf), "(none)");
+   sbprintf(buf, sizeof(buf), "(none)");
    sc_console_write_line(SC_CONSOLE(adg->console_guidance), 0, 0, buf);
    sc_active_console_add_row_spot(SC_ACTIVE_CONSOLE(adg->console_guidance), 0, NULL);
 
@@ -306,7 +322,7 @@ void _sc_window_auto_defense_guidance_init(sc_auto_def_gtk *adg) {
    for(count = 0; count < adg->set_guidance.length; ++count) {
       /* Generate a name tag for the item in question. */
       info = sc_accessory_lookup(ac, adg->set_guidance.items[count], SC_ACCESSORY_LIMIT_ALL);
-      snprintfn(buf, sizeof(buf), "%s (%02i)", info->name, info->inventories[adg->p->index]);
+      sbprintf(buf, sizeof(buf), "%s (%02i)", info->name, info->inventories[adg->p->index]);
       /* Display is offset +1 from index because of the ``(none)'' item. */
       sc_console_write_line(SC_CONSOLE(adg->console_guidance), 0, count + 1, buf);
       sc_active_console_add_row_spot(SC_ACTIVE_CONSOLE(adg->console_guidance), count + 1, NULL);
@@ -317,13 +333,20 @@ void _sc_window_auto_defense_guidance_init(sc_auto_def_gtk *adg) {
     * just enter_spot.  Can you think of others that would be useful?
     * I can't.  :)
     */
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_guidance), "enter-spot",
-                            GTK_SIGNAL_FUNC(_sc_autodef_guidance_enter_gtk), adg);
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_guidance), "key_press_spot",
-                            GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #if GTK12_ENABLED
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_guidance), "enter-spot",
+                               GTK_SIGNAL_FUNC(_sc_autodef_guidance_enter_gtk), adg);
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_guidance), "key_press_spot",
+                               GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #else /* GTK 2.0 */
+      g_signal_connect_after(G_OBJECT(adg->console_guidance), "enter-spot",
+                             (GCallback)_sc_autodef_guidance_enter_gtk, adg);
+      g_signal_connect_after(G_OBJECT(adg->console_guidance), "key_press_spot",
+                             (GCallback)_sc_window_auto_def_key_gtk, adg);
+   #endif /* GTK version? */
 
    /* Display the guidance system window. */
-   sc_display_console_attach(SC_DISPLAY(adg->w->screen), SC_CONSOLE(adg->console_guidance));
+   sc_display_console_attach(SC_DISPLAY(w->screen), SC_CONSOLE(adg->console_guidance));
 
 }
 
@@ -336,7 +359,8 @@ void _sc_window_auto_defense_shield_init(sc_auto_def_gtk *adg) {
 
    int count;
    char buf[SC_GTK_STRING_BUFFER];
-   sc_accessory_config *ac = adg->w->c->accessories;
+   sc_window_gtk *w = adg->w;
+   sc_accessory_config *ac = w->c->accessories;
    sc_accessory_info *info;
 
    /* Make a title on the main window. */
@@ -362,11 +386,11 @@ void _sc_window_auto_defense_shield_init(sc_auto_def_gtk *adg) {
    adg->set_shield.items = (int *)realloc(adg->set_shield.items, adg->set_shield.length * sizeof(int));
 
    /* Create the shield selection window. */
-   adg->console_shield = sc_active_console_new(45, 9, 32, 3, CONSOLE_NORMAL);
+   adg->console_shield = sc_window_active_console_new(w, 45, 9, 32, 3, CONSOLE_NORMAL);
    sc_console_buffer_size(SC_CONSOLE(adg->console_shield), 32, adg->set_shield.length + 1);
 
    /* Draw the ``nothing selected'' item and hook it up. */
-   snprintfn(buf, sizeof(buf), "(none)");
+   sbprintf(buf, sizeof(buf), "(none)");
    sc_console_write_line(SC_CONSOLE(adg->console_shield), 0, 0, buf);
    sc_active_console_add_row_spot(SC_ACTIVE_CONSOLE(adg->console_shield), 0, NULL);
 
@@ -374,7 +398,7 @@ void _sc_window_auto_defense_shield_init(sc_auto_def_gtk *adg) {
    for(count = 0; count < adg->set_shield.length; ++count) {
       /* Generate a name tag for the item in question. */
       info = sc_accessory_lookup(ac, adg->set_shield.items[count], SC_ACCESSORY_LIMIT_ALL);
-      snprintfn(buf, sizeof(buf), "%s (%02i)", info->name, info->inventories[adg->p->index]);
+      sbprintf(buf, sizeof(buf), "%s (%02i)", info->name, info->inventories[adg->p->index]);
       /* Display is offset +1 from index because of the ``(none)'' item. */
       sc_console_write_line(SC_CONSOLE(adg->console_shield), 0, count + 1, buf);
       sc_active_console_add_row_spot(SC_ACTIVE_CONSOLE(adg->console_shield), count + 1, NULL);
@@ -385,13 +409,20 @@ void _sc_window_auto_defense_shield_init(sc_auto_def_gtk *adg) {
     * just enter_spot.  Can you think of others that would be useful?
     * I can't.  :)
     */
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_shield), "enter-spot",
-                            GTK_SIGNAL_FUNC(_sc_autodef_shield_enter_gtk), adg);
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_shield), "key_press_spot",
-                            GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #if GTK12_ENABLED
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_shield), "enter-spot",
+                               GTK_SIGNAL_FUNC(_sc_autodef_shield_enter_gtk), adg);
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_shield), "key_press_spot",
+                               GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #else /* GTK 2.0 */
+      g_signal_connect_after(G_OBJECT(adg->console_shield), "enter-spot",
+                             (GCallback)_sc_autodef_shield_enter_gtk, adg);
+      g_signal_connect_after(G_OBJECT(adg->console_shield), "key_press_spot",
+                             (GCallback)_sc_window_auto_def_key_gtk, adg);
+   #endif /* GTK version? */
 
    /* Display the shield selection window. */
-   sc_display_console_attach(SC_DISPLAY(adg->w->screen), SC_CONSOLE(adg->console_shield));
+   sc_display_console_attach(SC_DISPLAY(w->screen), SC_CONSOLE(adg->console_shield));
 
 }
 
@@ -433,8 +464,10 @@ void _sc_window_auto_defense_triggers_init(sc_auto_def_gtk *adg) {
    Draw up the console pane for Contact Trigger settings.
    There is allocation here so don't call it multiple times. */
 
+   sc_window_gtk *w = adg->w;
+
    /* Create the contact triggers window. */
-   adg->console_triggers = sc_active_console_new(17, 16, 16, 5, CONSOLE_BORDERLESS);
+   adg->console_triggers = sc_window_active_console_new(w, 17, 16, 16, 5, CONSOLE_BORDERLESS);
 
    sc_console_write_line(SC_CONSOLE(adg->console_triggers),
                          0, 0, "Contact Triggers");
@@ -451,11 +484,16 @@ void _sc_window_auto_defense_triggers_init(sc_auto_def_gtk *adg) {
       gtk_widget_set_sensitive(GTK_WIDGET(adg->console_triggers), FALSE);
 
    /* Handle signals; most are handled natively by the toggle... */
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_triggers), "key_press_spot",
-                            GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #if GTK12_ENABLED
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_triggers), "key_press_spot",
+                               GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+   #else /* GTK 2.0 */
+      g_signal_connect_after(G_OBJECT(adg->console_triggers), "key_press_spot",
+                             (GCallback)_sc_window_auto_def_key_gtk, adg);
+   #endif /* GTK version? */
 
    /* Display the contact triggers window. */
-   sc_display_console_attach(SC_DISPLAY(adg->w->screen), SC_CONSOLE(adg->console_triggers));
+   sc_display_console_attach(SC_DISPLAY(w->screen), SC_CONSOLE(adg->console_triggers));
 
 }
 
@@ -483,20 +521,20 @@ void sc_window_auto_defense(sc_window *w_, sc_player *p) {
 
    /*   H E A D E R   */
 
-   /* Make a new window.                      x   y   w   h */
-   adg->console_main = sc_active_console_new( 8,  4, 70, 20, CONSOLE_NORMAL);
+   /* Make a new window.                               x   y   w   h */
+   adg->console_main = sc_window_active_console_new(w, 8,  4, 70, 20, CONSOLE_NORMAL);
    sc_display_console_attach(SC_DISPLAY(w->screen), SC_CONSOLE(adg->console_main));
    sc_console_set_foreground(SC_CONSOLE(adg->console_main), &w->colormap->pcolors[p->index]);
    gtk_widget_set_sensitive(GTK_WIDGET(adg->console_main), FALSE);
 
    /* Put a highlighted title on the new window. */
-   snprintfn(buf, sizeof(buf), "Tank Defense Controller for %s", p->name);
+   sbprintf(buf, sizeof(buf), "Tank Defense Controller for %s", p->name);
    sc_console_write_line(SC_CONSOLE(adg->console_main),
-                         70 / 2 - 14 - strlen(p->name) / 2, 0, buf);
+                         70 / 2 - 14 - strlenn(p->name) / 2, 0, buf);
    sc_console_highlight_attach(SC_CONSOLE(adg->console_main),
                                NULL, NULL, TRUE,
-                               70 / 2 - 14 - strlen(p->name) / 2, 0,
-                               28 + strlen(p->name), 1);
+                               70 / 2 - 14 - strlenn(p->name) / 2, 0,
+                               28 + strlenn(p->name), 1);
 
 
    /*   S E L E C T I O N S   */
@@ -521,7 +559,7 @@ void sc_window_auto_defense(sc_window *w_, sc_player *p) {
    /*   F O O T E R   */
 
    /* Float the exit instructions on their own console to fix null-focus crap. */
-   adg->console_activate = sc_active_console_new(8, 23, 70, 1, CONSOLE_BORDERLESS);
+   adg->console_activate = sc_window_active_console_new(w, 8, 23, 70, 1, CONSOLE_BORDERLESS);
 
    /* Display the EXIT instructions. */
    sc_console_write_line(SC_CONSOLE(adg->console_activate), 0, 0,
@@ -532,10 +570,17 @@ void sc_window_auto_defense(sc_window *w_, sc_player *p) {
    sc_active_console_add_spot(SC_ACTIVE_CONSOLE(adg->console_activate), 58, 0, 12, 1, NULL);
 
    /* Connect the signal handlers for EXIT conditions. */
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_activate), "key_press_spot",
-                            GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
-   gtk_signal_connect_after(GTK_OBJECT(adg->console_activate), "select-spot",
-                            GTK_SIGNAL_FUNC(_sc_window_auto_def_close_gtk), adg);
+   #if GTK12_ENABLED
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_activate), "key_press_spot",
+                               GTK_SIGNAL_FUNC(_sc_window_auto_def_key_gtk), adg);
+      gtk_signal_connect_after(GTK_OBJECT(adg->console_activate), "select-spot",
+                               GTK_SIGNAL_FUNC(_sc_window_auto_def_close_gtk), adg);
+   #else /* GTK 2.0 */
+      g_signal_connect_after(G_OBJECT(adg->console_activate), "key_press_spot",
+                             (GCallback)_sc_window_auto_def_key_gtk, adg);
+      g_signal_connect_after(G_OBJECT(adg->console_activate), "select-spot",
+                             (GCallback)_sc_window_auto_def_close_gtk, adg);
+   #endif /* GTK version? */
 
    sc_display_console_attach(SC_DISPLAY(w->screen), SC_CONSOLE(adg->console_activate));
 
