@@ -1,17 +1,16 @@
-/* $Header: /fridge/cvs/xscorch/sgtk/sexplosion-gtk.c,v 1.3 2001/07/14 21:17:33 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgtk/sexplosion-gtk.c,v 1.12 2004/02/26 06:34:55 justins Exp $ */
 /*
    
-   xscorch - sexplosion-gtk.c Copyright(c) 2001,2000 Justin David Smith
+   xscorch - sexplosion-gtk.c Copyright(c) 2000-2003 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
     
    Window painting code, specific to the explosion cache
-   I can't believe I wrote this mess...'
+   I can't believe I wrote this mess...
     
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,17 +25,19 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <xscorch.h>
+
+#include <sgtk.h>
 #include <sdisplay.h>
-#include <swindow-gtk.h>
-#include <sexplosion-gtk.h>
+
 #include <scolor-gtk.h>
+#include <sexplosion-gtk.h>
 #include <smenu-gtk.h>
-#include <sutil/sfractal.h>
+#include <swindow-gtk.h>
+
+#include <sgame/sland.h>
 #include <sgame/splayer.h>
 #include <sgame/sweapon.h>
-#include <sgame/sland.h>
-#include <sutil/sstr.h>
+#include <sutil/sfractal.h>
 
 
 
@@ -100,14 +101,25 @@ void sc_expl_cache_free_gtk(sc_expl_cache_gtk **cache) {
    
    /* Release pixmaps and bitmaps in cache */
    for(i = 0; i < (*cache)->cachesize; ++i) {
-      gdk_pixmap_unref((*cache)->cache[(*cache)->headptr].pixmap);
+      #if GTK12_ENABLED
+         gdk_pixmap_unref((*cache)->cache[(*cache)->headptr].pixmap);
+      #else /* GTK 2.0 */
+         g_object_unref((*cache)->cache[(*cache)->headptr].pixmap);
+      #endif /* GTK version? */
       ++(*cache)->headptr;
-      if((*cache)->headptr >= SC_EXPL_CACHE_SIZE) (*cache)->headptr = 0;
+      if((*cache)->headptr >= SC_EXPL_CACHE_SIZE) {
+         (*cache)->headptr = 0;
+      }
    } /* Releasing all pixmaps, bitmaps... */
    
    /* Release the GC */
-   gdk_gc_unref((*cache)->bitmapgc);
-   gdk_bitmap_unref((*cache)->fakebitmap);
+   #if GTK12_ENABLED
+      gdk_gc_unref((*cache)->bitmapgc);
+      gdk_bitmap_unref((*cache)->fakebitmap);
+   #else /* GTK 2.0 */
+      g_object_unref((*cache)->bitmapgc);
+      g_object_unref((*cache)->fakebitmap);
+   #endif /* GTK version? */
    
    /* Release the cache memory */
    free(*cache);
@@ -117,7 +129,8 @@ void sc_expl_cache_free_gtk(sc_expl_cache_gtk **cache) {
 
 
 
-static inline void _sc_expl_cache_draw_points_gtk(GdkImage *image, guint32 pixel, int dx, int dy, int radius) {
+static inline void _sc_expl_cache_draw_points_gtk(GdkImage *image, guint32 pixel, 
+                                                  int dx, int dy, int radius) {
 /* sc_expl_cache_draw_points_gtk
    Draw the points (cx +/- dx, cy +/- dy), using the GC's specified.  */
    
@@ -131,7 +144,9 @@ static inline void _sc_expl_cache_draw_points_gtk(GdkImage *image, guint32 pixel
 
 
 
-static inline void _sc_expl_cache_annihilate_column_rad2_gtk(GdkImage *image, GdkColor *gradient, int gsize, int dx, int dy, int radius, int rad2) {
+static inline void _sc_expl_cache_annihilate_column_rad2_gtk(GdkImage *image, GdkColor *gradient, 
+                                                             int gsize, int dx, int dy, 
+                                                             int radius, int rad2) {
 /* sc_expl_cache_annihilate_column_rad2_gtk
    Annihilate a column of an explosion.  (cx,cy) is the center coordinate of
    the explosion.  (dx,dy) is the deltax and deltay that this column begins
@@ -169,7 +184,8 @@ static inline void _sc_expl_cache_annihilate_column_rad2_gtk(GdkImage *image, Gd
 
 
 
-static inline void _sc_expl_cache_annihilate_column_rad_gtk(GdkImage *image, GdkColor *gradient, int gsize, int dx, int dy, int radius) {
+static inline void _sc_expl_cache_annihilate_column_rad_gtk(GdkImage *image, GdkColor *gradient,
+                                                            int gsize, int dx, int dy, int radius) {
 /* sc_expl_cache_annihilate_column_rad_gtk
    Annihilate a column of an explosion.  (cx,cy) is the center coordinate of
    the explosion.  (dx,dy) is the deltax and deltay that this column begins
@@ -216,7 +232,9 @@ static inline void _sc_expl_cache_annihilate_column_rad_gtk(GdkImage *image, Gdk
 
 
 
-static inline void _sc_expl_cache_annihilate_column_plasmoid_gtk(GdkImage *image, GdkColor *gradient, int gsize, int dx, int dy, int radius, unsigned char *fractal, int fsize) {
+static inline void _sc_expl_cache_annihilate_column_plasmoid_gtk(GdkImage *image, GdkColor *gradient,
+                                                                 int gsize, int dx, int dy, int radius,
+                                                                 unsigned char *fractal, int fsize) {
 /* sc_expl_cache_annihilate_column_plasmoid_gtk
    Annihilate a column of an explosion.  (cx,cy) is the center coordinate of
    the explosion.  (dx,dy) is the deltax and deltay that this column begins
@@ -227,26 +245,36 @@ static inline void _sc_expl_cache_annihilate_column_plasmoid_gtk(GdkImage *image
    int dx2y2;        /* dx^2 * y^2, for current value of y */
    int tile;         /* Tile value for explosion */
    int y;            /* Current y (offset from cy, y in [0,dy]) */
+   int radminusdx;   /* Value of radius - dx */
+   int radplusdx;    /* Value of radius + dx */
+
+   /* Precompute as much as possible. */
+   radminusdx = radius - dx;
+   radplusdx  = radius + dx;
 
    /* Iteration for all four columns */
    y = 0;            /* We are starting on the Y axis */
    dx2y2 = dx * dx;  /* This starts off as x^2, since y=0 */
    do {
+      /* Warning: the division must remain inlined with the rest of the
+         computation; it cannot be lifted out since this is integer-level
+         arithmetic. */
+   
       /* Calculate tile code (quadrant 1) */
-      tile = *(fractal + (radius + y) * fsize + (radius + dx)) * gsize / 0x100;
-      gdk_image_put_pixel(image, radius + dx, radius + y, gradient[tile].pixel);
+      tile = *(fractal + (radius + y) * fsize + radplusdx)  * gsize / 0x100;
+      gdk_image_put_pixel(image, radplusdx,  radius + y, gradient[tile].pixel);
 
       /* Calculate tile code (quadrant 2) */
-      tile = *(fractal + (radius + y) * fsize + (radius - dx)) * gsize / 0x100;
-      gdk_image_put_pixel(image, radius - dx, radius + y, gradient[tile].pixel);
+      tile = *(fractal + (radius + y) * fsize + radminusdx) * gsize / 0x100;
+      gdk_image_put_pixel(image, radminusdx, radius + y, gradient[tile].pixel);
 
       /* Calculate tile code (quadrant 3) */
-      tile = *(fractal + (radius - y) * fsize + (radius - dx)) * gsize / 0x100;
-      gdk_image_put_pixel(image, radius - dx, radius - y, gradient[tile].pixel);
+      tile = *(fractal + (radius - y) * fsize + radminusdx) * gsize / 0x100;
+      gdk_image_put_pixel(image, radminusdx, radius - y, gradient[tile].pixel);
 
       /* Calculate tile code (quadrant 4) */
-      tile = *(fractal + (radius - y) * fsize + (radius + dx)) * gsize / 0x100;
-      gdk_image_put_pixel(image, radius + dx, radius - y, gradient[tile].pixel);
+      tile = *(fractal + (radius - y) * fsize + radplusdx)  * gsize / 0x100;
+      gdk_image_put_pixel(image, radplusdx,  radius - y, gradient[tile].pixel);
    
       /* Adjust dx2y2 for next value of y */
       /* Suggested by JJP */
@@ -260,7 +288,8 @@ static inline void _sc_expl_cache_annihilate_column_plasmoid_gtk(GdkImage *image
 
 
 
-static void _sc_expl_cache_annihilate_rad2_gtk(GdkImage *image, GdkColor *gradient, int gsize, int radius) {
+static void _sc_expl_cache_annihilate_rad2_gtk(GdkImage *image, GdkColor *gradient, 
+                                               int gsize, int radius) {
 /* sc_expl_annihilate_rad2_gtk
    Annihilate a region centered at (cx,cy) for a radius r.  This function
    lets colors fall off with the square of the radius.  */
@@ -278,7 +307,7 @@ static void _sc_expl_cache_annihilate_rad2_gtk(GdkImage *image, GdkColor *gradie
    rad2major2 = rad2;/* Radius^2 + major^2, running total */
    min2thresh = rad2 - dy; /* Minimum threshold before need to redraw edges */
    
-   /* TEMP */
+   /* TEMP HACK */
    /* This is a hack to prevent the orange highlights on explosions. */
    /* THIS IS A HACK -- FIXME FOR REAL. */
    rad2 += radius + radius + 1;
@@ -288,11 +317,13 @@ static void _sc_expl_cache_annihilate_rad2_gtk(GdkImage *image, GdkColor *gradie
       be filling parts of the circle multiple times.  Hence the 
       min2thresh variable. */
    do {
-      _sc_expl_cache_annihilate_column_rad2_gtk(image, gradient, gsize, dx, dy, radius, rad2);
+      _sc_expl_cache_annihilate_column_rad2_gtk(image, gradient, gsize, 
+                                                dx, dy, radius, rad2);
       ++dx;
       rad2major2 -= (dx << 1) - 1;
       if(rad2major2 <= min2thresh) {
-         _sc_expl_cache_annihilate_column_rad2_gtk(image, gradient, gsize, dy, dx, radius, rad2);
+         _sc_expl_cache_annihilate_column_rad2_gtk(image, gradient, gsize, 
+                                                   dy, dx, radius, rad2);
          --dy;
          min2thresh -= (dy << 1);
       }
@@ -302,7 +333,8 @@ static void _sc_expl_cache_annihilate_rad2_gtk(GdkImage *image, GdkColor *gradie
 
 
 
-static void _sc_expl_cache_annihilate_rad_gtk(GdkImage *image, GdkColor *gradient, int gsize, int radius) {
+static void _sc_expl_cache_annihilate_rad_gtk(GdkImage *image, GdkColor *gradient, 
+                                              int gsize, int radius) {
 /* sc_expl_annihilate_rad_gtk
    Annihilate a region centered at (cx,cy) for a radius r.  This function
    lets colors fall off with the square of the radius.  */
@@ -325,11 +357,13 @@ static void _sc_expl_cache_annihilate_rad_gtk(GdkImage *image, GdkColor *gradien
       be filling parts of the circle multiple times.  Hence the 
       min2thresh variable. */
    do {
-      _sc_expl_cache_annihilate_column_rad_gtk(image, gradient, gsize, dx, dy, radius);
+      _sc_expl_cache_annihilate_column_rad_gtk(image, gradient, gsize, 
+                                               dx, dy, radius);
       ++dx;
       rad2major2 -= (dx << 1) - 1;
       if(rad2major2 <= min2thresh) {
-         _sc_expl_cache_annihilate_column_rad_gtk(image, gradient, gsize, dy, dx, radius);
+         _sc_expl_cache_annihilate_column_rad_gtk(image, gradient, gsize, 
+                                                  dy, dx, radius);
          --dy;
          min2thresh -= (dy << 1);
       }
@@ -339,7 +373,8 @@ static void _sc_expl_cache_annihilate_rad_gtk(GdkImage *image, GdkColor *gradien
 
 
 
-static void _sc_expl_cache_annihilate_plasmoid_gtk(GdkImage *image, GdkColor *gradient, int gsize, int radius) {
+static void _sc_expl_cache_annihilate_plasmoid_gtk(GdkImage *image, GdkColor *gradient,
+                                                   int gsize, int radius) {
 /* sc_expl_annihilate_plasmoid_gtk
    Annihilate a region centered at (cx,cy) for a radius r.  This function
    selects colors to form a plasma manifold pattern, whatever that means.  */
@@ -352,7 +387,7 @@ static void _sc_expl_cache_annihilate_plasmoid_gtk(GdkImage *image, GdkColor *gr
    unsigned char *fractal;     /* */
    int fsize;
 
-   /* */
+   /* Allocate a new fractal. */
    fractal = sc_fractal_create(&fsize, radius + radius + 1);
    if(fractal == NULL) {
       _sc_expl_cache_annihilate_rad2_gtk(image, gradient, gsize, radius);
@@ -371,11 +406,13 @@ static void _sc_expl_cache_annihilate_plasmoid_gtk(GdkImage *image, GdkColor *gr
       be filling parts of the circle multiple times.  Hence the 
       min2thresh variable. */
    do {
-      _sc_expl_cache_annihilate_column_plasmoid_gtk(image, gradient, gsize, dx, dy, radius, fractal, fsize);
+      _sc_expl_cache_annihilate_column_plasmoid_gtk(image, gradient, gsize, 
+                                                    dx, dy, radius, fractal, fsize);
       ++dx;
       rad2major2 -= (dx << 1) - 1;
       if(rad2major2 <= min2thresh) {
-         _sc_expl_cache_annihilate_column_plasmoid_gtk(image, gradient, gsize, dy, dx, radius, fractal, fsize);
+         _sc_expl_cache_annihilate_column_plasmoid_gtk(image, gradient, gsize, 
+                                                       dy, dx, radius, fractal, fsize);
          --dy;
          min2thresh -= (dy << 1);
       }
@@ -387,13 +424,17 @@ static void _sc_expl_cache_annihilate_plasmoid_gtk(GdkImage *image, GdkColor *gr
 
 
 
-static inline void _sc_expl_cache_annihilate_happy_gtk(sc_window_gtk *w, GdkPixmap *pixmap, int radius) {
-/* ? */
+static inline void _sc_expl_cache_annihilate_happy_gtk(sc_window_gtk *w, GdkPixmap *pixmap, 
+                                                       int radius) {
+/* sc_expl_cache_annihilate_happy_gtk
+   It's not a bug... it's a ``feature''...  I wonder if anyone has actually
+   found this egg in practice.  It's quite fun once it's activated :)  */
 
    int size = radius + radius + 1;
    int eyesize = (radius >> 2) + (radius >> 4);
    int eyecoff = radius >> 2;
    GdkGC *gc = gdk_gc_new(pixmap);
+
    gdk_gc_set_foreground(gc, &w->colormap->yellow);
    gdk_draw_arc(pixmap, gc, TRUE, 0, 0, size, size, 0, 360 * 64);
    gdk_gc_set_foreground(gc, &w->colormap->black);
@@ -408,70 +449,61 @@ static inline void _sc_expl_cache_annihilate_happy_gtk(sc_window_gtk *w, GdkPixm
    gdk_draw_arc(pixmap, gc, FALSE,
                 radius - eyecoff - eyesize, radius + (eyecoff >> 1),
                 (eyecoff + eyesize) << 1, eyesize << 1, 200 * 64, 140 * 64);
-   gdk_gc_unref(gc);
+   #if GTK12_ENABLED
+      gdk_gc_unref(gc);
+   #else /* GTK 2.0 */
+      g_object_unref(gc);
+   #endif /* GTK version? */
 
 }
 
 
 
-static inline void _sc_expl_cache_annihilate_gtk(sc_window_gtk *w, GdkPixmap *pixmap, int radius, sc_explosion_type type) {
+static inline void _sc_expl_cache_annihilate_gtk(sc_window_gtk *w, GdkPixmap *pixmap, 
+                                                 int radius, sc_explosion_type type) {
 /* sc_expl_annihilate_rad_gtk
    Annihilate a region centered at (cx,cy) for a radius r.  This function
    uses the type parameter to dispatch to the appropriate drawing
    subfunction.  */
 
    GdkImage *image;
-#ifdef __TEMP_obs
-   GdkImage *mask;
-   void *buffer;
-#endif
    int size;
 
+   /* ...wonder why this is here... :) */
    if(w->state < 4) {
       /* Create images */
       size = radius + radius + 1;
       image = gdk_image_new(GDK_IMAGE_FASTEST, gtk_widget_get_visual(w->app), size, size);
       if(image == NULL) return;
       
-   #ifdef __TEMP_obs
-      /* Create mask image */
-      buffer = malloc(SQR(size));
-      if(buffer == NULL) {
-         gdk_image_destroy(image);
-         return;
-      }
-      memsetn(buffer, 0xff, SQR(size));
-      mask = gdk_image_new_bitmap(gtk_widget_get_visual(w->app), buffer, size, size);
-      if(mask == NULL) {
-         free(buffer);
-         gdk_image_destroy(image);
-         return;
-      } /* Make sure image, mask were created */
-   #endif
-      
       /* Draw the appropriate type of explosion. */
       switch(type) {
          case SC_EXPLOSION_NORMAL:
-            _sc_expl_cache_annihilate_rad2_gtk(image, w->colormap->gradient[SC_GRAD_EXPLOSION], w->c->colors->gradsize[SC_GRAD_EXPLOSION], radius);
+            _sc_expl_cache_annihilate_rad2_gtk(image, w->colormap->gradient[SC_GRAD_EXPLOSION], 
+                                               w->c->colors->gradsize[SC_GRAD_EXPLOSION], radius);
             break;
          case SC_EXPLOSION_PLASMA:
-            _sc_expl_cache_annihilate_plasmoid_gtk(image, w->colormap->gradient[SC_GRAD_EXPLOSION], w->c->colors->gradsize[SC_GRAD_EXPLOSION], radius);
+            _sc_expl_cache_annihilate_plasmoid_gtk(image, w->colormap->gradient[SC_GRAD_EXPLOSION], 
+                                                   w->c->colors->gradsize[SC_GRAD_EXPLOSION], radius);
             break;
          case SC_EXPLOSION_SPIDER:
-            _sc_expl_cache_annihilate_rad_gtk(image, w->colormap->gradient[SC_GRAD_FUNKY_EXPLOSION], w->c->colors->gradsize[SC_GRAD_FUNKY_EXPLOSION], radius);
+            _sc_expl_cache_annihilate_rad_gtk(image, w->colormap->gradient[SC_GRAD_FUNKY_EXPLOSION],
+                                              w->c->colors->gradsize[SC_GRAD_FUNKY_EXPLOSION], radius);
             break;
          default:
+            /* do nothing */;
       } /* Which type of explosion to draw? */
 
       /* Commit data to pixmap/bitmap buffers */
-      gdk_draw_image(pixmap, sc_display_get_gc(SC_DISPLAY(w->screen)), image, 0, 0, 0, 0, size, size);
+      gdk_draw_image(pixmap, sc_display_get_gc(SC_DISPLAY(w->screen)), 
+                     image, 0, 0, 0, 0, size, size);
       
       /* Destroy the local images */
-      gdk_image_destroy(image);
-   #ifdef __TEMP_obs
-      gdk_image_destroy(mask);
-      /* danger: do not attempt free(buffer); */
-   #endif
+      #if GTK12_ENABLED
+         gdk_image_destroy(image);
+      #else /* GTK 2.0 */
+         g_object_unref(image);
+      #endif /* GTK version? */
    } else {
       _sc_expl_cache_annihilate_happy_gtk(w, pixmap, radius);
    }
@@ -532,6 +564,9 @@ int sc_expl_cache_new(sc_window *w_, int radius, sc_explosion_type type) {
    if(cacheid < 0) {
       /* No matching explosion found; overwrite the oldest
          entry in the cache with our new explosion.  */
+      /* TEMP BUG:  We really shouldn't overwrite the OLDEST
+         entry; we'd get better performance if we overwrite
+         the LEAST-RECENTLY-USED entry.  Need to fix this... */
       --cache->headptr;
       if(cache->headptr < 0) cache->headptr = SC_EXPL_CACHE_SIZE - 1;
       cacheid = cache->headptr;
@@ -543,7 +578,11 @@ int sc_expl_cache_new(sc_window *w_, int radius, sc_explosion_type type) {
          ++cache->cachesize;
       } else {
          /* Cache was full; need to release the old pixmaps */
-         gdk_pixmap_unref(centry->pixmap);
+         #if GTK12_ENABLED
+            gdk_pixmap_unref(centry->pixmap);
+         #else /* GTK 2.0 */
+            g_object_unref(centry->pixmap);
+         #endif /* GTK version? */
       } /* Was cache already full? */
 
       /* Create the new pixmap and bitmap, and set characteristics */
@@ -586,21 +625,37 @@ void sc_expl_cache_draw(sc_window *w_, int ptr, int centerx, int centery, int ra
    mask = gdk_pixmap_new(NULL, size, size, 1);
    if(mask != NULL) {
       gdk_gc_set_foreground(w->explcache->bitmapgc, &black);
-      gdk_draw_rectangle(mask, w->explcache->bitmapgc, TRUE, 0, 0, size, size);
+      gdk_draw_rectangle(mask, w->explcache->bitmapgc, TRUE, 
+                         0, 0, size, size);
       gdk_gc_set_foreground(w->explcache->bitmapgc, &white);
-      gdk_draw_arc(mask, w->explcache->bitmapgc, TRUE, 0, 0, size, size, 0, 360 * 64);
+      gdk_draw_arc(mask, w->explcache->bitmapgc, TRUE, 
+                   0, 0, size, size, 0, 360 * 64);
    }
          
    /* Draw the explosion to the screen buffer */
    gdk_gc_set_clip_mask(sc_display_get_gc(SC_DISPLAY(w->screen)), mask);
-   gdk_gc_set_clip_origin(sc_display_get_gc(SC_DISPLAY(w->screen)), centerx - rad, centery - rad);
-   gdk_draw_pixmap(sc_display_get_buffer(SC_DISPLAY(w->screen)), sc_display_get_gc(SC_DISPLAY(w->screen)), centry->pixmap, radius - rad, radius - rad, centerx - rad, centery - rad, size, size);
+   gdk_gc_set_clip_origin(sc_display_get_gc(SC_DISPLAY(w->screen)), 
+                          centerx - rad, centery - rad);
+   gdk_draw_drawable(sc_display_get_buffer(SC_DISPLAY(w->screen)),
+                     sc_display_get_gc(SC_DISPLAY(w->screen)),
+                     centry->pixmap,
+                     radius - rad, radius - rad,
+                     centerx - rad, centery - rad,
+                     size, size);
    gdk_gc_set_clip_mask(sc_display_get_gc(SC_DISPLAY(w->screen)), NULL);
 
    /* Update drawing */   
-   sc_display_queue_draw(SC_DISPLAY(w->screen), centerx - rad, centery - rad, size, size);
+   sc_display_queue_draw(SC_DISPLAY(w->screen),
+                         centerx - rad, centery - rad,
+                         size, size);
    
    /* Uninstall the bitmap */
-   if(mask != NULL) gdk_bitmap_unref(mask);
+   if(mask != NULL) {
+      #if GTK12_ENABLED
+         gdk_bitmap_unref(mask);
+      #else /* GTK 2.0 */
+         g_object_unref(mask);
+      #endif /* GTK version? */
+   }
    
 }
