@@ -1,7 +1,7 @@
-/* $Header: /fridge/cvs/xscorch/sgtk/simage-gtk.h,v 1.2 2001/04/07 19:44:17 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgtk/simage-gtk.h,v 1.4 2004/02/26 06:34:55 justins Exp $ */
 /*
    
-   xscorch - simage-gtk.h     Copyright(c) 2001,2000 Justin David Smith
+   xscorch - simage-gtk.h     Copyright(c) 2000-2003 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
     
    GTK interface to image drawing
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +25,7 @@
 #define __simage_gtk_h_included
 
 
+#include <sgtk.h>
 #include <gtk/gtk.h>
 
 
