@@ -1,7 +1,7 @@
-/* $Header: /fridge/cvs/xscorch/sgtk/snet-gtk.h,v 1.2 2001/04/07 19:44:17 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgtk/snet-gtk.h,v 1.4 2004/02/26 06:34:55 justins Exp $ */
 /*
    
-   xscorch - snet-gtk.h       Copyright(c) 2001,2000 Justin David Smith
+   xscorch - snet-gtk.h       Copyright(c) 2000-2003 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
     
    Function prototypes for network windows
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +26,6 @@
 
 
 #include <swindow-gtk.h>
-#include <xscorch.h>
 #include <snet/snet.h>
 
 
