@@ -1,7 +1,7 @@
-/* $Header: /fridge/cvs/xscorch/sgtk/spaint-gtk.c,v 1.8 2001/07/08 18:06:35 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sgtk/spaint-gtk.c,v 1.15 2004/02/26 06:34:55 justins Exp $ */
 /*
    
-   xscorch - spain-gtk.c      Copyright(c) 2001,2000 Justin David Smith
+   xscorch - spain-gtk.c      Copyright(c) 2000-2003 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
     
    Window painting code for scorch
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,20 +24,23 @@
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <xscorch.h>
-#include <sdisplay.h>
+
+#include <sgtk.h>
 #include <sconsole.h>
-#include <swindow-gtk.h>
+#include <sdisplay.h>
+
 #include <scolor-gtk.h>
-#include <smenu-gtk.h>
 #include <simage-gtk.h>
+#include <smenu-gtk.h>
+#include <swindow-gtk.h>
+
 #include <sgame/saccessory.h>
-#include <sgame/stankpro.h>
-#include <sgame/sphysics.h>
-#include <sgame/sshield.h>
-#include <sgame/splayer.h>
-#include <sgame/sweapon.h>
 #include <sgame/sland.h>
+#include <sgame/sphysics.h>
+#include <sgame/splayer.h>
+#include <sgame/sshield.h>
+#include <sgame/stankpro.h>
+#include <sgame/sweapon.h>
 
 
 
@@ -50,7 +52,7 @@
 
 
 
-static inline int _sc_lines_overlap_gtk(int a1, int a2, int b1, int b2) {
+static inline bool _sc_lines_overlap_gtk(int a1, int a2, int b1, int b2) {
 /* sc_lines_overlap_gtk */
 
    /* Check if line segment (a1,a2) overlaps (b1,b2) */
@@ -60,17 +62,20 @@ static inline int _sc_lines_overlap_gtk(int a1, int a2, int b1, int b2) {
 
 
 
-static inline int _sc_rects_overlap_gtk(int ax1, int ay1, int ax2, int ay2, int bx1, int by1, int bx2, int by2) {
+static inline bool _sc_rects_overlap_gtk(int ax1, int ay1, int ax2, int ay2,
+                                         int bx1, int by1, int bx2, int by2) {
 /* sc_rects_overlap_gtk */
 
    /* Check if rectangle A overlaps any part of rectanble B. */
-   return(_sc_lines_overlap_gtk(ax1, ax2, bx1, bx2) && _sc_lines_overlap_gtk(ay1, ay2, by1, by2));
+   return(_sc_lines_overlap_gtk(ax1, ax2, bx1, bx2) && 
+          _sc_lines_overlap_gtk(ay1, ay2, by1, by2));
 
 }
 
 
 
-static void _sc_window_draw_arrow(GdkPixmap *buffer, GdkGC *gc, int x, int y, int size, bool right) {
+static void _sc_window_draw_arrow(GdkPixmap *buffer, GdkGC *gc, int x, int y,
+                                  int size, bool right) {
 /* sc_window_draw_arrow
    Draws an arrow on the screen whose RIGHT coordinate of the mainline
    is at (x, y), and whose mainline is <size> pixels long.  The <right>
@@ -82,11 +87,15 @@ static void _sc_window_draw_arrow(GdkPixmap *buffer, GdkGC *gc, int x, int y, in
 
    /* Draw the arrow cap */
    if(right) {
-      gdk_draw_line(buffer, gc, x, y, x - SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
-      gdk_draw_line(buffer, gc, x, y, x - SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
+      gdk_draw_line(buffer, gc, x, y,
+                    x - SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
+      gdk_draw_line(buffer, gc, x, y,
+                    x - SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
    } else {
-      gdk_draw_line(buffer, gc, x - size, y, x - size + SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
-      gdk_draw_line(buffer, gc, x - size, y, x - size + SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
+      gdk_draw_line(buffer, gc, x - size, y, 
+                    x - size + SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
+      gdk_draw_line(buffer, gc, x - size, y, 
+                    x - size + SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
    }
 
 }
@@ -100,8 +109,10 @@ static void _sc_window_draw_x(GdkPixmap *buffer, GdkGC *gc, int x, int y) {
    coordinates MUST be real coordiantes.  */
 
    /* Draw the X */
-   gdk_draw_line(buffer, gc, x, y + SC_WIND_ARROW_SIZE, x - SC_WIND_ARROW_SIZE - SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
-   gdk_draw_line(buffer, gc, x, y - SC_WIND_ARROW_SIZE, x - SC_WIND_ARROW_SIZE - SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
+   gdk_draw_line(buffer, gc, x, y + SC_WIND_ARROW_SIZE,
+                 x - SC_WIND_ARROW_SIZE - SC_WIND_ARROW_SIZE, y - SC_WIND_ARROW_SIZE);
+   gdk_draw_line(buffer, gc, x, y - SC_WIND_ARROW_SIZE,
+                 x - SC_WIND_ARROW_SIZE - SC_WIND_ARROW_SIZE, y + SC_WIND_ARROW_SIZE);
 
 }
 
@@ -162,10 +173,10 @@ static void _sc_window_draw_wind_arrow(sc_window_gtk *w, bool drawing) {
    } else {
       /* Clearing an existing arrow */
       if(size < SC_WIND_MINIMUM_ARROW) size = SC_WIND_ARROW_SIZE + SC_WIND_ARROW_SIZE;
-      gdk_draw_pixmap(buffer, gc, w->landbuffer, 
-                      x - size, y - SC_WIND_ARROW_SIZE,
-                      x - size, y - SC_WIND_ARROW_SIZE,
-                      size + 3, SC_WIND_ARROW_SIZE + SC_WIND_ARROW_SIZE + 3);
+      gdk_draw_drawable(buffer, gc, w->landbuffer, 
+                        x - size, y - SC_WIND_ARROW_SIZE,
+                        x - size, y - SC_WIND_ARROW_SIZE,
+                        size + 3, SC_WIND_ARROW_SIZE + SC_WIND_ARROW_SIZE + 3);
    } /* Drawing or clearing? */
    
    /* Update the display */
@@ -185,7 +196,7 @@ static void _sc_window_draw_tank_gtk(sc_window_gtk *w, const sc_player *p) {
    GdkPixmap *buffer;         /* Screen buffer */
    GdkGC *gc;                 /* Screen gc */
    sc_gradient_list grad;     /* Gradient for shields */
-   char *data;                /* Profile data pointer */
+   unsigned char *data;       /* Profile data pointer */
    int radius;                /* Tank's radius */
    int index;                 /* Shield gradient index */
    int size;                  /* Size = radius * 2 + 1 */
@@ -250,10 +261,12 @@ static void _sc_window_draw_tank_gtk(sc_window_gtk *w, const sc_player *p) {
          if(*data != SC_TANK_PROFILE_CLEAR) {
             tx = x + cx;
             ty = y + cy;
-            if(sc_land_translate_x(w->c->land, &tx)) gdk_draw_point(buffer, gc, tx, ty);
-         }
-      }
-   }
+            if(sc_land_translate_x(w->c->land, &tx)) {
+               gdk_draw_point(buffer, gc, tx, ty);
+            } /* Is this coordinate on the screen? */
+         } /* Is this point part of the tank? */
+      } /* Iterate over X */
+   } /* Iterate over Y */
 
    /* Draw tank's turret. */
    radius = p->tank->turretradius;
@@ -464,7 +477,11 @@ static void _sc_window_draw_land_image_gtk(sc_window_gtk *w, int x1, int y1, int
    /* Copy local image to the offscreen pixmap */
    gdk_draw_image(w->landbuffer, sc_display_get_gc(SC_DISPLAY(w->screen)), image, 
                   0, 0, x1, fheight - y2, width, height);
-   gdk_image_destroy(image);
+   #if GTK12_ENABLED
+      gdk_image_destroy(image);
+   #else /* GTK 2.0 */
+      g_object_unref(image);
+   #endif /* GTK version? */
    
 }
 
@@ -480,8 +497,8 @@ static inline void _sc_window_draw_land_gtk(sc_window_gtk *w, int x1, int y1, in
    /* Make sure X, Y are in bounds. */
    if(x1 < 0) x1 = 0;
    if(y1 < 0) y1 = 0;
-   if(x2 >= w->c->fieldwidth) x2 = w->c->fieldwidth - 1;
-   if(y2 >= w->c->fieldheight)y2 = w->c->fieldheight- 1;
+   if(x2 >= w->c->fieldwidth)  x2 = w->c->fieldwidth - 1;
+   if(y2 >= w->c->fieldheight) y2 = w->c->fieldheight- 1;
 
    /* Which subfunction to use? */
    if((y2 - y1) * (x2 - x1) <= USE_PIXMAP_MAX_AREA) { 
@@ -528,7 +545,9 @@ void sc_window_paint(sc_window *w_, int x1, int y1, int x2, int y2, int flags) {
    /* Check for overlap */
    ox1 = x1;
    ox2 = x2;
-   if(sc_land_overlap_x(w->c->land, &ox1, &ox2)) sc_window_paint(w_, ox1, y1, ox2, y2, flags);
+   if(sc_land_overlap_x(w->c->land, &ox1, &ox2)) {
+      sc_window_paint(w_, ox1, y1, ox2, y2, flags);
+   }
     
    /* Determine the screen boundaries. */
    screenx = x1;
@@ -548,8 +567,10 @@ void sc_window_paint(sc_window *w_, int x1, int y1, int x2, int y2, int flags) {
 
    /* Copy current land pixmap to screen. */
    if(flags & SC_REDRAW_LAND) {
-      gdk_draw_pixmap(sc_display_get_buffer(SC_DISPLAY(w->screen)), sc_display_get_gc(SC_DISPLAY(w->screen)), w->landbuffer, 
-                      screenx, screeny, screenx, screeny, screenw, screenh);
+      gdk_draw_drawable(sc_display_get_buffer(SC_DISPLAY(w->screen)), 
+                        sc_display_get_gc(SC_DISPLAY(w->screen)),
+                        w->landbuffer, 
+                        screenx, screeny, screenx, screeny, screenw, screenh);
       sc_display_queue_draw(SC_DISPLAY(w->screen), screenx, screeny, screenw, screenh);
    } /* Copy land to screen? */
 
@@ -620,8 +641,13 @@ void sc_window_paint_circular(sc_window *w_, int centerx, int centery, int rad,
    
    /* Uninstall the clipping mask */
    gdk_gc_set_clip_mask(displaygc, NULL);
-   if(mask != NULL) gdk_bitmap_unref(mask);
-   if(gc != NULL) gdk_gc_unref(gc);
+   #if GTK12_ENABLED
+      if(mask != NULL) gdk_bitmap_unref(mask);
+      if(gc != NULL) gdk_gc_unref(gc);
+   #else /* GTK 2.0 */
+      if(mask != NULL) g_object_unref(mask);
+      if(gc != NULL) g_object_unref(gc);
+   #endif /* GTK version? */
    
 }
 
@@ -713,10 +739,16 @@ void sc_window_paint_blank(sc_window *w_) {
 
    sc_window_gtk *w = (sc_window_gtk *)w_;
 
-   gdk_gc_set_foreground(sc_display_get_gc(SC_DISPLAY(w->screen)), &w->colormap->black);
-   gdk_draw_rectangle(sc_display_get_buffer(SC_DISPLAY(w->screen)), sc_display_get_gc(SC_DISPLAY(w->screen)), TRUE, 
-                      0, 0, w->screen->allocation.width, w->screen->allocation.height);
-   sc_display_queue_draw(SC_DISPLAY(w->screen), 0, 0, w->screen->allocation.width, w->screen->allocation.height);
+   gdk_gc_set_foreground(sc_display_get_gc(SC_DISPLAY(w->screen)),
+                         &w->colormap->black);
+   gdk_draw_rectangle(sc_display_get_buffer(SC_DISPLAY(w->screen)),
+                      sc_display_get_gc(SC_DISPLAY(w->screen)),
+                      TRUE,
+                      0, 0,
+                      w->screen->allocation.width, w->screen->allocation.height);
+   sc_display_queue_draw(SC_DISPLAY(w->screen),
+                         0, 0,
+                         w->screen->allocation.width, w->screen->allocation.height);
 
 }
 
@@ -733,7 +765,9 @@ static sc_trajectory_result _sc_window_draw_arc_gtk(sc_config *c, sc_trajectory
 
    x = rint(tr->curx);
    y = rint(tr->cury);
-   if(!sc_land_translate_xy(w->c->land, &x, &y)) return(SC_TRAJ_CONTINUE);
+   if(!sc_land_translate_xy(w->c->land, &x, &y)) {
+      return(SC_TRAJ_CONTINUE);
+   }
    
    buffer = sc_display_get_buffer(SC_DISPLAY(w->screen));   
    gc = sc_display_get_gc(SC_DISPLAY(w->screen));
@@ -746,7 +780,7 @@ static sc_trajectory_result _sc_window_draw_arc_gtk(sc_config *c, sc_trajectory
 
 
 
-static sc_trajectory_result _sc_window_clear_arc_gtk(sc_config *c, sc_trajectory *tr, void *data) {
+static sc_trajectory_result _sc_window_clear_arc_gtk(unused sc_config *c, sc_trajectory *tr, void *data) {
 /* sc_window_clear_arc_gtk */
 
    sc_window_gtk *w = (sc_window_gtk *)data;
