@@ -1,8 +1,8 @@
-/* $Header: /fridge/cvs/xscorch/snet/snetclient.c,v 1.18 2001/08/15 05:34:35 jacob Exp $ */
+/* $Header: /fridge/cvs/xscorch/snet/snetclient.c,v 1.25 2004/02/26 06:34:56 justins Exp $ */
 /*
 
-   xscorch - snetclient.c     Copyright(c) 2001,2000 Justin David Smith
-                              Copyright(c) 2001      Jacob Luna Lundberg
+   xscorch - snetclient.c     Copyright(c) 2000-2003 Justin David Smith
+                              Copyright(c) 2001-2003 Jacob Luna Lundberg
    justins(at)chaos2.org      http://chaos2.org/
    jacob(at)chaos2.org        http://chaos2.org/~jacob
 
@@ -11,8 +11,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,28 +23,25 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
-#define  __ALLOW_STRING_H__
+#include <errno.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+
 #include <snetint.h>
 
-
-
-#include <sgame/sgame.h>
-#include <sgame/sstate.h>
-#include <sgame/swindow.h>
-#include <sgame/sinventory.h>
 #include <sgame/saccessory.h>
-#include <sgame/sweapon.h>
+#include <sgame/sgame.h>
+#include <sgame/sinventory.h>
 #include <sgame/splayer.h>
 #include <sgame/sshield.h>
+#include <sgame/sstate.h>
+#include <sgame/sweapon.h>
+#include <sgame/swindow.h>
 #include <sutil/srand.h>
-#include <sutil/sstr.h>
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
+#include <libj/jstr/libjstr.h>
 
 
 
@@ -143,7 +139,7 @@ static bool _sc_net_cli_recv_sync(sc_config *c, sc_client *cli, sc_packet *packe
 
 
 
-static bool _sc_net_cli_recv_orders(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_orders(sc_config *c, sc_packet *packet) {
 /* _sc_net_cli_recv_orders
    Receiving orders from another player in the game. We are given our
    client data structure, and their packet (as relayed from the server).
@@ -207,14 +203,14 @@ static bool _sc_net_cli_recv_orders(sc_config *c, sc_client *cli, sc_packet *pac
 
 
 
-static bool _sc_net_cli_recv_shields(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_shields(sc_config *c, sc_packet *packet) {
 /* _sc_net_cli_recv_shields
    Similar to the orders packet, except we are receiving a player request
    to activate their shields. Again, returns true on success. */
 
    sc_accessory_info *info;/* Shield requested for activation */
    sc_player *pl;          /* Player structure */
-   dword playerid;         /* Player ID to shield */
+   sdword playerid;        /* Player ID to shield */
    dword *p;               /* counter */
 
    /* Check the packet size */
@@ -252,13 +248,13 @@ static bool _sc_net_cli_recv_shields(sc_config *c, sc_client *cli, sc_packet *pa
 
 
 
-static bool _sc_net_cli_recv_battery(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_battery(sc_config *c, sc_packet *packet) {
 /* _sc_net_cli_recv_battery
    Similar to the orders packet, except we are receiving a player request
    to activate one of their batteries. Again, returns true on success. */
 
    sc_player *pl;          /* Player structure */
-   dword playerid;         /* Player ID to shield */
+   sdword playerid;        /* Player ID to shield */
 
    /* Check the packet size */
    if(!sc_net_check_size( packet, 
@@ -287,13 +283,13 @@ static bool _sc_net_cli_recv_battery(sc_config *c, sc_client *cli, sc_packet *pa
 
 
 
-static bool _sc_net_cli_recv_flags(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_flags(sc_config *c, sc_packet *packet) {
 /* _sc_net_cli_recv_flags
    Similar to the orders packet, except we are receiving a player's
    set of command flags.  Again, returns true on success. */
 
    sc_player *pl;          /* Player structure */
-   dword playerid;         /* Player ID to shield */
+   sdword playerid;        /* Player ID to shield */
    dword *p;               /* Packet data pointer */
 
    /* Check the packet size */
@@ -336,12 +332,12 @@ static bool _sc_net_cli_recv_flags(sc_config *c, sc_client *cli, sc_packet *pack
 
 
 
-static bool _sc_net_cli_recv_inventory(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_inventory(sc_config *c, sc_packet *packet) {
 
    sc_accessory_info *ai;  /* A temp pointer into info structs */
    sc_weapon_info *wi;     /* A temp pointer into info structs */
    sc_player *pl;          /* Player structure */
-   dword playerid;         /* Player ident */
+   sdword playerid;        /* Player ident */
    dword *p;               /* Pointer into packet data */
    int acount;             /* Accessory count (total) */
    int wcount;             /* Weapon count (total) */
@@ -445,12 +441,13 @@ static bool _sc_net_cli_recv_inventory(sc_config *c, sc_client *cli, sc_packet *
 
 
 
-static bool _sc_net_cli_recv_player_state(sc_config *c, sc_client *cli, sc_packet *packet) {
+static bool _sc_net_cli_recv_player_state(sc_config *c, sc_packet *packet) {
 
    sc_player *pl;          /* Player data structure */
    dword *p;               /* Pointer into packet data */
-   dword x;                /* Miscellaneous variable X */
-   dword y;                /* Miscellaneous variable Y */
+   sdword x;               /* Miscellaneous variable X */
+   sdword y;               /* Miscellaneous variable Y */
+   bool b;                 /* Miscellaneous boolean B */
    int i;                  /* Iterator variable */
 
    /* Check that the packet is large enough */
@@ -491,11 +488,11 @@ static bool _sc_net_cli_recv_player_state(sc_config *c, sc_client *cli, sc_packe
             sc_player_set_power(c, pl, y);
          }
          x = ntohl(*p++);
-         y = ntohl(*p++);
-         if(x != pl->life || y != pl->dead) {
+         b = ntohl(*p++);
+         if(x != pl->life || b != pl->dead) {
             sc_net_set_error("cli_recv_player_state", "Discrepancy on remote player life");
             pl->life = x;
-            pl->dead = y;
+            pl->dead = b;
          }
          x = ntohl(*p++);
          if(x != pl->money) {
@@ -525,8 +522,8 @@ static bool _sc_net_cli_recv_player_state(sc_config *c, sc_client *cli, sc_packe
             sc_net_set_error("cli_recv_player_state", "Discrepancy on remote player turret/power");
          }
          x = ntohl(*p++);
-         y = ntohl(*p++);
-         if(x != pl->life || y != pl->dead) {
+         b = ntohl(*p++);
+         if(x != pl->life || b != pl->dead) {
             sc_net_set_error("cli_recv_player_state", "Discrepancy on remote player life");
          }
          x = ntohl(*p++);
@@ -559,12 +556,12 @@ static bool _sc_net_process_message(sc_config *c, sc_client *cli, sc_packet *pac
       _sc_net_client_send_name(cli);
       memcpy(buffer, packet->data + sizeof(dword), packet->data_size - sizeof(dword));
       buffer[packet->data_size - sizeof(dword)] = '\0';
-      snprintfn(infomsg, sizeof(infomsg), "We have connected to server, version \"%s\"", buffer);
+      sbprintf(infomsg, sizeof(infomsg), "We have connected to server, version \"%s\"", buffer);
       sc_net_set_info("process_message", infomsg);
       break;
    case SC_NET_SVR_REJECTED:
       SC_CONN_SET_FLAGS(cli->server, SC_CONN_REJECTED);
-      snprintfn(infomsg, sizeof(infomsg), "We were rejected from server:  %s", packet->data);
+      sbprintf(infomsg, sizeof(infomsg), "We were rejected from server:  %s", packet->data);
       sc_net_set_error("process_message", infomsg);
       sc_window_message(c->window, "Client Rejected", sc_net_get_error());
       break;
@@ -573,8 +570,8 @@ static bool _sc_net_process_message(sc_config *c, sc_client *cli, sc_packet *pac
       sc_net_set_info("process_message", "client: server sent QUIT");
       break;
    case SC_NET_CHAT:
-      memcpy(buffer, packet->data, MIN(packet->data_size, SC_NET_BUFFER_SIZE - 1));
-      buffer[MIN(packet->data_size, SC_NET_BUFFER_SIZE - 1)] = '\0';
+      memcpy(buffer, packet->data, min(packet->data_size, SC_NET_BUFFER_SIZE - 1));
+      buffer[min(packet->data_size, SC_NET_BUFFER_SIZE - 1)] = '\0';
       sc_net_set_info("chat", buffer);
       sc_chat_window_update(c->window, buffer);
       break;
@@ -585,22 +582,22 @@ static bool _sc_net_process_message(sc_config *c, sc_client *cli, sc_packet *pac
       sc_net_cli_recv_players(c, cli, packet);
       break;
    case SC_NET_SVR_ORDERS:
-      _sc_net_cli_recv_orders(c, cli, packet);
+      _sc_net_cli_recv_orders(c, packet);
       break;
    case SC_NET_INVENTORY:
-      _sc_net_cli_recv_inventory(c, cli, packet);
+      _sc_net_cli_recv_inventory(c, packet);
       break;
    case SC_NET_SHIELDS:
-      _sc_net_cli_recv_shields(c, cli, packet);
+      _sc_net_cli_recv_shields(c, packet);
       break;
    case SC_NET_BATTERY:
-      _sc_net_cli_recv_battery(c, cli, packet);
+      _sc_net_cli_recv_battery(c, packet);
       break;
    case SC_NET_PLFLAGS:
-      _sc_net_cli_recv_flags(c, cli, packet);
+      _sc_net_cli_recv_flags(c, packet);
       break;
    case SC_NET_PLAYER_STATE:
-      _sc_net_cli_recv_player_state(c, cli, packet);
+      _sc_net_cli_recv_player_state(c, packet);
       break;
    case SC_NET_SVR_SYNC_RESP:
       _sc_net_cli_recv_sync(c, cli, packet);
@@ -609,7 +606,7 @@ static bool _sc_net_process_message(sc_config *c, sc_client *cli, sc_packet *pac
       sc_net_client_recv_status(c, cli, packet);
       break;
    default:
-      snprintfn(infomsg, sizeof(infomsg), "invalid packet type (%08x) received by client", packet->msg_type);
+      sbprintf(infomsg, sizeof(infomsg), "invalid packet type (%08x) received by client", packet->msg_type);
       sc_net_set_error("process_message", infomsg);
       break;
    }
@@ -627,6 +624,14 @@ bool sc_net_client_handle_packet(sc_config *c, void *parm, sc_packet *p) {
    /* Sanity checks */
    if(c == NULL || parm == NULL || p == NULL) return(false);
 
+   /* Compare RNG state with that of the server... */
+   if(p->next_rnd != game_rand_peek()) {
+      /* TEMP - This area needs a more graceful failure path
+                and a better error message in general. */
+      printf("warning: server says our next random value is %i but we thought it was %i\n",
+             p->next_rnd, game_rand_peek());
+   }
+
    /* This is an existing connection, process as event */
    return(_sc_net_process_message(c, cli, p));
 
@@ -672,7 +677,7 @@ bool sc_net_client_chat(sc_client *cli, const char *msg) {
    if(cli == NULL) return(false);
 
    if(!SC_CONN_IS_DEAD(cli->server)) {
-      snprintfn(buffer, sizeof(buffer), "%s: %s", cli->name, msg);
+      sbprintf(buffer, sizeof(buffer), "%s: %s", cli->name, msg);
       return(sc_net_send_message(&cli->server, SC_NET_CHAT, buffer));
    }
    return(false);
