@@ -1,16 +1,15 @@
-/* $Header: /fridge/cvs/xscorch/sutil/srand.h,v 1.2 2001/04/07 19:49:00 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sutil/srand.h,v 1.5 2004/02/26 06:34:57 justins Exp $ */
 /*
-   
+
    xscorch - srand.h          Copyright(c) 2000 Justin David Smith
    justins(at)chaos2.org      http://chaos2.org/
 
    Scorched random numbers
-    
+
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,6 +24,8 @@
 #ifndef __srand_h_included
 #define __srand_h_included
 
+#include <xscorch.h>
+
 
 /* Random functions */
 void     sys_randomize(void);
