@@ -1,4 +1,4 @@
-/* $Header: /fridge/cvs/xscorch/sutil/sslink.h,v 1.2 2001/04/07 19:49:00 justins Exp $ */
+/* $Header: /fridge/cvs/xscorch/sutil/sslink.h,v 1.4 2004/02/26 06:34:57 justins Exp $ */
 /*
 
    xscorch - sslink.h         Copyright(c) 2000 Jacob Luna Lundberg
@@ -9,8 +9,7 @@
 
    This program is free software; you can redistribute it and/or modify 
    it under the terms of the GNU General Public License as published by 
-   the Free Software Foundation; either version 2 of the License, or 
-   (at your option) any later version.
+   the Free Software Foundation, version 2 of the License ONLY. 
 
    This program is distributed in the hope that it will be useful, 
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,8 +21,8 @@
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 */
-#ifndef __sslist_h_included
-#define __sslist_h_included
+#ifndef __sslink_h_included
+#define __sslink_h_included
 
 
 
@@ -42,4 +41,4 @@ slinklist *slink_append(slinklist *head, slinklist *item);
 
 
 
-#endif /* __sslist_h_included */
+#endif /* __sslink_h_included */
